#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#
# v3.0~mickyz

import dbus, gobject, gtk, gettext, locale, optparse, sys
from gui import mainWindow
from tools import consts, loadGladeFile, prefs

# command line
optparser = optparse.OptionParser(usage='decibel [option] [file1 file2...]')
optparser.add_option('--new', action='store_true', default=False,
    help='start a new instance')
(optOptions, optArgs) = optparser.parse_args()

# check whether is already running
if not optOptions.new:
    shouldStop  = False
    dbusSession = None

    try:
        dbusSession    = dbus.SessionBus()
        activeServices = dbusSession.get_object(
            'org.freedesktop.DBus',
            '/org/freedesktop/DBus').ListNames()

        if consts.dbusService in activeServices:
            shouldStop = True
            # fill current instance with given tracks
            if len(optArgs) != 0:
                dbus.Interface(
                    dbusSession.get_object(consts.dbusService, '/TrackList'),
                    consts.dbusInterface).SetTracks(optArgs, True)
    except:
        pass

    if dbusSession is not None:
        dbusSession.close()

    if shouldStop:
        sys.exit(1)

prefs.setCmdLine((optOptions, optArgs))

# localization
locale.setlocale(locale.LC_ALL, '')
gettext.textdomain(consts.appNameShort)
gettext.bindtextdomain(consts.appNameShort, consts.dirLocale)
gtk.glade.textdomain(consts.appNameShort)
gtk.glade.bindtextdomain(consts.appNameShort, consts.dirLocale)

print '\x1b[32;01m[INFO]\x1b[0m', 'Initializing...'

# pygtk initialization
gobject.threads_init()
gtk.window_set_default_icon_list(
    consts.appIcon16,
    consts.appIcon24,
    consts.appIcon32,
    consts.appIcon48,
    consts.appIcon96)

# main window
wtree  = loadGladeFile('MainWindow.glade')
window = wtree.get_object('win-main')
prefs.setWidgetsTree(wtree)

# rgba support
try:
    colormap = window.get_screen().get_rgba_colormap()
    if colormap:
        gtk.widget_set_default_colormap(colormap)
except:
    pass

# this object takes care of the window (mainly event handlers)
mainWindow.MainWindow(wtree, window)

def delayedStartup():
    """ Perform all initialization stuff that is not mandatory to display window,
        should be called within the GTK main loop, once window has been displayed
    """
    import atexit, dbus.mainloop.glib, modules, signal

    def atExit():
        """ Final function, called just before exiting the Python interpreter """
        prefs.save()
        print '\x1b[31;01m[QUIT]\x1b[0m', 'Bye...'

    def onInterrupt(window):
        """ Handler for interrupt signals e.g., Ctrl-C """
        window.hide()
        modules.postQuitMsg()

    # d-bus mainloop
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    # register handlers
    atexit.register(atExit)
    # interrupt signal
    signal.signal(signal.SIGINT,  lambda sig, frame: onInterrupt(window))
    signal.signal(signal.SIGTERM, lambda sig, frame: onInterrupt(window))
    # start all modules
    gobject.idle_add(modules.postMsg, consts.MSG_EVT_APP_STARTED)

# come on baby, light my sound ^_^
gobject.idle_add(delayedStartup)
gtk.main()
