# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz #FIXME

import gtk
from tools import prefs

WIN_WIDTH  = 480
WIN_HEIGHT = 480

class MainWindow:

    def __init__(self, wtree, window):
        """ Restore the state of the window """
        #if prefs.get(__name__, 'win-is-maximized', False): window.maximize()
        winWidth = prefs.get(__name__, 'win-width', WIN_WIDTH)
        winHeight = prefs.get(__name__, 'win-height', WIN_HEIGHT)
        window.resize(winWidth, winHeight)
        window.set_position(gtk.WIN_POS_CENTER)#FIXME save/restore position
        window.connect('delete-event', self.onQuit)
        # objects to hide
        objects = ('menu-bar', 'scrolled-trklist', 'hbox-trklist', 'status-bar')
        self.objects = ([wtree.get_object(i) for i in objects])
        # resize events
        event = wtree.get_object('evt-resize')
        event.connect('button-press-event', self.doResize, window)
        #self.signal = self.event.connect(.. #self.event.disconnect(self.signal)
        menuitem = wtree.get_object('menu-show')
        menuitem.connect('button-press-event', lambda w,e:
            event.emit('button-press-event', gtk.gdk.Event(gtk.gdk.BUTTON_PRESS)))
        notebook = wtree.get_object('notebook-main')
        notebook.connect('switch-page', self.__doResize, event)
        # menu items
        wtree.get_object('menu-about').connect('activate', lambda i:
            self.menuAbout(window))
        wtree.get_object('menu-preferences').connect('activate',
            self.menuPrefs)
        wtree.get_object('menu-quit').connect('activate', lambda i:
            self.menuQuit(window))

        window.show_all()

    def doResize(self, widget, event, win):
        """ Resize window at left click """
        if win.window.get_state() == gtk.gdk.WINDOW_STATE_MAXIMIZED:
            return
        #f event.button == 1: # left-click
            #pass
        if event.button == 2: # middle-click
            if self.objects[0].get_visible():
                win.resize(WIN_WIDTH, WIN_HEIGHT)
            return
        if event.button == 3: # rigth-click
            event.button = 0  #
        if event.button == 0: # set_visible(True/False)
            if self.objects[0].get_visible():
                self.win_width, self.win_height = win.get_size()
                for i in self.objects: i.hide()
                win.resize(self.win_width, 1)
            else:
                for i in self.objects: i.show()
                win.resize(self.win_width, self.win_height)

    def __doResize(self, widget, page, page_num, event):
        """ Restore to full view at click on tabs """
        if page_num != 0 and not self.objects[0].get_visible():
            event.emit('button-press-event', gtk.gdk.Event(gtk.gdk.BUTTON_PRESS))

    def saveState(self, win):
        """ Save the state of the window """
        if not win.window: return

        win_is_max = win.window.get_state() & gtk.gdk.WINDOW_STATE_MAXIMIZED
        if not win_is_max:
            win_width, win_height = win.get_size()
            if win_width < WIN_WIDTH or win_height < WIN_HEIGHT:
                win_width, win_height = WIN_WIDTH, WIN_HEIGHT
            prefs.set(__name__, 'win-width', win_width)
            prefs.set(__name__, 'win-height', win_height)
        #prefs.set(__name__, 'win-is-maximized', bool(win_is_max))

        win.hide()

    def onQuit(self, win, event):
        """ Quit at click on close button """
        import modules
        
        for i in modules.getModules(): # deny close clicks
            if (i[0] == 'StatusIcon' and i[1][2] != None):
                win.iconify()
                return True

        self.saveState(win)

        modules.postQuitMsg()
        return True

    def menuQuit(self, win):
        """ Quit """
        import modules

        self.saveState(win)

        modules.postQuitMsg()

    def menuPrefs(self, item):
        """ Preferences """
        import modules

        modules.showPreferences()

    def menuAbout(self, win):
        """ About """
        import gui.about

        gui.about.show(win)
