# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v2.9~mickyz

import gtk, tools

class Window(gtk.Window):
    """ Add some functionalities to gtk.Window:
        * Add a isVisible() function
        * Add a getWidget() function that acts like get_object() in gtk.glade
        * Hide the window instead of destroying it
    """
    def __init__(self, resFile, container, parent, title, width, height):
        """ Constructor """
        gtk.Window.__init__(self)

        wTree, self.wBuilder = tools.loadGladeFile(resFile, container)
        self.wBuilder.get_object(container).reparent(self)
        self.resize(width, height)
        self.set_transient_for(parent)
        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
        self.set_title(title)
        self.visible = False

        self.connect('delete-event', self.delete)

    def getWidget(self, name):
        """ Return the widget with the given name """
        return self.wBuilder.get_object(name)

    def isVisible(self):
        """ Return True if the window is currently visible """
        return self.visible

    def show(self):
        """ Show the window if not visible, bring it to top otherwise """
        self.visible = True
        self.show_all()
        self.present()

    def hide(self):
        """ Hide the window """
        self.visible = False
        gtk.Window.hide(self)

    def delete(self, win, evt):
        """ Hide the window instead of deleting it """
        self.hide()
        return True
