# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz #FIXME

import os

from os.path import basename, dirname, isdir, isfile, join, splitext

from format import monkeysaudio, asf, flac, mp3, mp4, mpc, ogg, wav, wavpack
from media.filetrack import FileTrack
#from media import mime
from tools import consts, pickleLoad, pickleSave

isfile
# {dirPath: {fileName: track}}
TRACK_INDEX = pickleLoad(consts.filePlayed) \
    if (os.path.exists(consts.filePlayed)) else {}

# supported formats with mutagen modules
TRACK_EXT = {
    '.ac3':  monkeysaudio,
    '.ape':  monkeysaudio,
    '.flac': flac,
    '.m4a':  mp4,
    '.mp2':  mp3,
    '.mp3':  mp3,
    '.mp4':  mp4,
    '.mpc':  mpc,
    '.oga':  ogg,
    '.ogg':  ogg,
    '.wav':  wav,
    '.wma':  asf,
    '.wv':   wavpack
    }

def isSupported(file):
    """ Return True if the given file is a supported format """
    try: return splitext(file.lower())[1] in TRACK_EXT.keys()
    except: return False #return mime.guess_supported(file)

def isSupportedPlaylist(file):
    """ Return True if the given file is a supported playlist """
    return file.lower()[-4:] == '.m3u'

def getSupportedFormats():
    """ Return a list of all formats from which tags can be extracted """
    return ['*' + ext for ext in TRACK_EXT]

def getTrackFromFile(file):
    """ Return a Track object based on the tags of the given file """
    try:
        return TRACK_EXT[splitext(file.lower())[1]].getTrack(file)
    except:
        return FileTrack(file)

def getTracksFromFiles(files):
    """ Same as getTrackFromFile(), but works on a list of files """
    return [getTrackFromFile(file) for file in files]

def getTracks(filenames, sortedByName=True, ignoreHiddenFiles=True):#FIXME
    """ Same as getTracksFromFiles(), but with walk in any sub directories """
    trackfiles = []

    for directory in [filename for filename in filenames if isdir(filename)]:
        for root, subdirs, files in os.walk(directory):
            for file in files:
               if not ignoreHiddenFiles or file[0] != '.':
                    if isSupported(file):
                        trackfiles.append(join(root, file))

    for trackfile in [filename for filename in filenames
        if isfile(filename) and isSupported(filename)]:
        trackfiles.append(trackfile)

    for playlist in [filename for filename in filenames
        if isfile(filename) and isSupportedPlaylist(filename)]:
            for trackfile in [track for track in loadPlaylist(playlist)
                if track not in trackfiles]:
                trackfiles.append(trackfile)

    return getTracksFromIndex(sorted(trackfiles))

def getTracksFromIndex(files):#FIXME
    """ Return the Track objects reading the index of played tracks """
    trkList  = []
    counter = 0
    for f in files:
        dname, fname = dirname(f), basename(f)
        try:
            track = TRACK_INDEX[dname][fname]
            #print 110, 1
        except:
            # if len(files) <= 100: getTrackFromFile(file)???
            track = FileTrack(f)
            #print 114, 0
            subIndex = TRACK_INDEX.setdefault(dname, {})
            TRACK_INDEX[dname][fname] = track

        trkList.append(track)

    #pickleSave(consts.filePlayed, TRACK_INDEX)

    return trkList

def setTrackToIndex(track, path):
    """ Update the index of the played tracks """
    if track and path:
        dname, fname = dirname(path), basename(path)
        subIndex = TRACK_INDEX.setdefault(dname, {})
        TRACK_INDEX[dname][fname] = track
    else:
        pickleSave(consts.filePlayed, TRACK_INDEX)


def getPlaylist():#FIXME
    """ Return the list of Track objects from the given playlist """
    #return getTracksFromPlayed(loadPlaylist(consts.filePlaylist))
    return pickleLoad(consts.filePlaylist)

def setPlaylist(tracks):#FIXME
    """ Return the list of Track objects from the given playlist """
    #savePlaylist([track.getFilePath() for track in tracks], consts.filePlaylist)
    pickleSave(consts.filePlaylist, tracks)


def loadPlaylist(file):
    """ Return the list of files loaded from the given playlist """
    if not isfile(consts.filePlaylist):
        return []

    input = open(file)
    files = [line for line in [line.strip() for line in input] \
        if len(line) != 0 and line[0] != '#']
    input.close()

    path = dirname(file)
    for i, file in enumerate(files):
        if not os.path.isabs(file):
            files[i] = join(path, file.replace('\\', '/'))

    return [file for file in files if isfile(file)]

def savePlaylist(files, file):
    """ Create a playlist with the given files """
    with open(file, 'w') as f:
        f.writelines('\n'.join(files))
        #f.write('#EXTM3U\n#EXTINF:\n')
        #f.writelines('\n#EXTINF:\n'.join(files))
