# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz

import gtk, modules, os, tools
from tools import consts, prefs, log
from gettext import gettext as _
# python-imaging module
import Image

MOD_INFO = ('Covers', _('Covers'), _('Download the album covers'),
    False, True, 1)
MOD_CACHE = os.path.join(consts.dirCfg, MOD_INFO[0])
if not os.path.exists(MOD_CACHE):
    os.mkdir(MOD_CACHE)

# audioscrobbler vars
KEY = 'fd8dd98d26bb3f288f3e626502f9add6'
URL = 'http://ws.audioscrobbler.com/2.0/?method='
USR = 'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/2008072820 Firefox/3.0.1'

# thumbnail vars
THUMB_FRAME = consts.pixPath('cover-frame.png')
THUMB_OVERLAY = consts.pixPath('cover-gloss.png')
THUMB_OFFSETX = 3 #12
THUMB_OFFSETY = 4 #3
THUMB_WIDTH = 90
THUMB_HEIGHT = 90

(
    PIL_WIDTH,
    PIL_HEIGHT,
) = range(2)

class Covers(modules.ThreadedModule):

    def __init__(self):
        """ Constructor """
        handlers = {
            consts.MSG_EVT_STOPPED:     self.onNewCover,
            consts.MSG_EVT_NEW_TRACK:   self.onNewTrack,
                   }
        modules.ThreadedModule.__init__(self, handlers)

        self.download = prefs.get(__name__, 'download', True)
        self.coverWidget = prefs.getWidgetsTree().get_object('img-cover')
        self.coverName = None
        self.cfgWin = None
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __pixThumbnail(self, file):
        """ Paste the cover on frame, add gloss effect and return the pixbuf """
        try:
            data = Image.open(file).convert('RGBA')
            # fit image into THUMB_WIDTH x THUMB_HEIGHT
            data = data.resize((THUMB_WIDTH, THUMB_HEIGHT), Image.ANTIALIAS)
            # paste resized image
            model = Image.open(THUMB_FRAME).convert('RGBA')
            model.paste(data, (THUMB_OFFSETX, THUMB_OFFSETY), data)
            data = model
            # apply gloss effect
            gloss = Image.open(THUMB_OVERLAY).convert('RGBA')
            data.paste(gloss, (0, 0), gloss)
            return consts.pixLoad(data)
        except Exception, err:
            log.c.error(err)
            return None

    def onNewTrack(self, track):
        """ A new track is being played, try to retrieve the cover """
        artist, album = track.getArtist().lower(), track.getAlbum().lower()
        # get hash so not request again
        coverName = '%s.jpg' % str(abs(hash(artist+album)))
        if coverName == self.coverName:
             return
        self.coverName = coverName
        # serach on current track directory
        fullSize = self.getFromUser(os.path.dirname(track.getFilePath()))
        # search on ~/.config/decibel cache
        if not fullSize: 
            for file in os.listdir(MOD_CACHE):
                if file == coverName:
                    fullSize = os.path.join(MOD_CACHE, coverName)
                    break
        # search on audioscrobbler, while waiting, if None set it
        self.onNewCover(fullSize)
        if not fullSize and self.download:
            fullSize = os.path.join(MOD_CACHE, coverName)
            data = self.getFromInternet(artist, album)
            if data:
                consts.pixSave(data, fullSize)
                self.onNewCover(fullSize)                    

    def onNewCover(self, fullSize=None):
        """ Generate the cover thumbnail to show """
        if fullSize: thumbnail = self.__pixThumbnail(fullSize)
        else: thumbnail = gtk.gdk.pixbuf_new_from_file(THUMB_FRAME)
        # set new cover on widget
        self.coverWidget.set_from_pixbuf(thumbnail)
        # MSG_EVT_NEW_COVER
        modules.postMsg(consts.MSG_EVT_NEW_COVER,
            {'pixThumbnail': thumbnail, 'pathFullSize': fullSize})
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __pixDownload(self, url):
        """ Download a picture from the given url, return the raw data """
        import urllib2, socket

        try:
            socket.setdefaulttimeout(2.0) # timeout in floating seconds
            request = urllib2.Request(url, headers = {'User-Agent': USR})
            stream = urllib2.urlopen(request)
            data = stream.read()
            return data
        except:
            return None

    def getFromInternet(self, artist, album):
        """ Return the downloaded album cover or None if except """
        import urllib2, socket

        socket.setdefaulttimeout(2.0) # timeout in floating seconds
        try:
            # request information for artist, album
            url = '%salbum.getinfo&artist=%s&album=%s&api_key=%s' % (
                URL,
                tools.percentEncode(artist),
                tools.percentEncode(album),
                KEY)
            request = urllib2.Request(url, headers = {'User-Agent': USR})
            stream = urllib2.urlopen(request)
            data = stream.read()
            # extract url
            startIdx = data.find('<image size="large">')
            endIdx = data.find('</image>', startIdx)
            pixURL = data[startIdx+len('<image size="large">'):endIdx]
            return self.__pixDownload(pixURL)
        except:
            pass #return None
        try:
            # if not found, request again but for artist only
            url = '%sartist.getimages&artist=%s&api_key=%s' % (
                URL,
                tools.percentEncode(artist),
                KEY)
            request = urllib2.Request(url, headers = {'User-Agent': USR})
            stream = urllib2.urlopen(request)
            data = stream.read()
            # extract url
            startswith = '<size name="largesquare" width="126" height="126">'
            startIdx = data.find(startswith)
            endIdx = data.find('</size>', startIdx)
            pixURL = data[startIdx+len(startswith):endIdx]
            return self.__pixDownload(pixURL)
        except:
            return None

    def getFromUser(self, dirpath):
        """ Return the filenames of  locally """
        candidates = {}
        for (file, path) in tools.listDir(dirpath, True):
            name, ext = os.path.splitext(file)
            if ext.lower() in ['.jpg', '.jpeg', '.png',]:
                candidates[name] = path
        for name in ['cd','cover','art','front','*']:
            if name in candidates:
                return candidates[name]
        if name == '*' and len(candidates) != 0:
            return candidates.values()[0]
        return None
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def configure(self, parent):
        """ Show configuration window """
        vBox = gtk.VBox()
        iBox = gtk.image_new_from_file(consts.pixPath('audioscrobbler.png'))
        cBox = gtk.CheckButton(_('Disable covers downloading'))
        cBox.set_active(not self.download)
        hBox = gtk.HBox()
        hBox.pack_start(cBox, True, True, 6)
        vBox.add(iBox)
        vBox.add(hBox)
        vBox.set_spacing(12)

        dialog = gtk.Dialog(MOD_INFO[1],
            prefs.getWidgetsTree().get_object('win-main'),
            gtk.DIALOG_MODAL, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
        dialog.get_content_area().pack_start(vBox, True, True, 6)
        dialog.set_default_size(240,1)
        dialog.show_all()

        response = dialog.run()

        self.download = not cBox.get_active()
        prefs.set(__name__, 'download', self.download)

        dialog.destroy()
