# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz#FIXME

import gtk, modules
from tools import consts, prefs, sec2str, htmlEscape

MOD_INFO = ('CtrlPanel', 'CtrlPanel', '', True, False, None)

class CtrlPanel(modules.Module):

    def __init__(self):
        """ Initialize the module """
        handlers = {
            consts.MSG_EVT_APP_STARTED:     self.onAppStarted,
            consts.MSG_EVT_APP_QUIT:        self.onAppQuit,
            consts.MSG_EVT_PAUSED:          self.onPaused,
            consts.MSG_EVT_STOPPED:         self.onStopped,
            consts.MSG_EVT_UNPAUSED:        self.onUnpaused,
            consts.MSG_EVT_NEW_TRACK:       self.onNewTrack,
            consts.MSG_EVT_TRACK_MOVED:     self.onCurrentTrackMoved,
            consts.MSG_EVT_TRACK_POSITION:  self.onNewTrackPosition,
            consts.MSG_EVT_NEW_TRACKLIST:   self.onNewTracklist,
                   }

        modules.Module.__init__(self, handlers)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onAppStarted(self):
        """ Initialization, called when module has been loaded """
        wTree          = prefs.getWidgetsTree()

        self.window    = wTree.get_object('win-main')

        self.lblTitle  = wTree.get_object('lbl-title')
        self.lblArtist = wTree.get_object('lbl-artist')
        self.lblInfo   = wTree.get_object('lbl-info')
        self.lblPos    = wTree.get_object('lbl-status1')
        self.lblFile   = wTree.get_object('lbl-status2')
        self.lblTime   = wTree.get_object('lbl-status3')

        self.btnPlay   = wTree.get_object('btn-play')
        self.btnNext   = wTree.get_object('btn-next')
        self.btnPrev   = wTree.get_object('btn-prev')
        self.sclSeek   = wTree.get_object('scl-position')
        self.btnVolume = wTree.get_object('btn-volume')
        #self.btnStop  = wTree.get_object('btn-stop')

        self.btnNext.connect('clicked',
            lambda widget: modules.postMsg(consts.MSG_CMD_NEXT))
        self.btnPrev.connect('clicked',
            lambda widget: modules.postMsg(consts.MSG_CMD_PREVIOUS))
        self.btnPlay.connect('clicked',
            lambda widget: modules.postMsg(consts.MSG_CMD_TOGGLE_PAUSE))
        #self.btnStop.connect('clicked',
        #   lambda widget: modules.postMsg(consts.MSG_CMD_STOP))
        self.sclDragged = False
        self.sclBeingDragged = False
        self.sclSeek.set_range(0, 100)
        self.sclSeek.set_increments(1, 10)

        value = prefs.get(__name__, 'volume', 1.0)#FIXME
        modules.postMsg(consts.MSG_CMD_SET_VOLUME, {'level': value})
        self.btnVolume.set_value(value)

        self.sclSeek.connect('change-value', self.onSeekChangingValue)
        self.sclSeek.connect('value-changed', self.onSeekValueChanged)
        self.sclSeek.connect('button-press-event', self.onSeekButtonPressed)
        self.sclSeek.connect('button-release-event', self.onSeekButtonReleased)

        self.btnVolume.connect('value-changed', self.onVolumeChanged)

    def onAppQuit(self):
        """ The application is about to terminate """
        prefs.set(__name__, 'volume', round(self.btnVolume.get_value(), 1))
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onNewTrack(self, track):
        """ A new track is being played """
        self.artist, self.title = track.getArtist(), track.getTitle()
        self.length = track.getLength()

        self.lblTitle.set_markup('<span size="larger"><b>%s </b></span>' %
            htmlEscape(self.title))

        if self.artist != '~':
            self.lblArtist.set_label('%s - %s' % (self.artist, track.getAlbum()))
            self.window.set_title('%s - %s' % (self.artist, self.title))
        else:
            self.lblArtist.set_label(track.getAlbum())
            self.window.set_title(self.title)

        self.lblInfo.set_markup('<span size="smaller">%s kpbs %s kHz</span>' %
            (track.getBitrate(), track.getSampleRate()))

        self.lblPos.set_label('#%s' % track.getPlaylistPos())

        self.lblTime.set_label('0:00/%s' % sec2str(self.length))

        if track.getURI().startswith('cdda:'):
            self.lblFile.set_label('%s' % self.title)
        else:  
            self.lblFile.set_label('%s' % track.getFileName())

        self.btnPlay.set_sensitive(True)
        self.btnPlay.set_image(gtk.image_new_from_pixbuf(consts.pauseIcon))
        #self.btnStop.set_sensitive(True)
        #print 116
        self.sclSeek.handler_block_by_func(self.onSeekValueChanged)
        self.sclSeek.set_range(0, int(self.length))
        self.sclSeek.set_sensitive(True)
        self.sclSeek.handler_unblock_by_func(self.onSeekValueChanged)

    def onNewTrackPosition(self, seconds):
        """ The track position has changed """        
        if not self.sclDragged:
            # make sure handler will not be called
            self.sclSeek.handler_block_by_func(self.onSeekValueChanged)
            self.sclSeek.set_value(seconds)
            self.sclSeek.handler_unblock_by_func(self.onSeekValueChanged)
            #print 129, seconds
            self.lblTime.set_label('%s/%s' % (
                sec2str(seconds), sec2str(self.length)))

    def onNewTracklist(self, tracks, playtime):
        """ A new tracklist has been set """
        self.tracks, self.playtime = len(tracks), playtime

        if self.lblFile.get_label() == '':   
            self.lblPos.set_label('#%s' % self.tracks
                if (playtime != 0) else '#')
            #print 141
            self.lblTime.set_label(sec2str(self.playtime)
                if (self.tracks != 0) else '0:00')

    def onStopped(self):
        """ The playback has been stopped """ 
        self.window.set_title(consts.appName)

        self.lblTitle.set_label(
            '<span size="larger"><b>%s </b></span>' % consts.appName)
        self.lblArtist.set_label(consts.appComment)
        self.lblInfo.set_label('')

        self.lblFile.set_label('')

        self.lblPos.set_label('#%s' % self.tracks
            if (self.tracks != 0) else '#')
        self.lblTime.set_label(sec2str(self.playtime)
            if (self.tracks != 0) else '0:00')

        self.btnNext.set_sensitive(False)
        self.btnPrev.set_sensitive(False)
        self.btnPlay.set_image(gtk.image_new_from_pixbuf(consts.playIcon))
        #self.btnStop.set_sensitive(False)
        #print 165
        self.sclSeek.handler_block_by_func(self.onSeekValueChanged)
        self.sclSeek.set_sensitive(False)
        self.sclSeek.set_value(0)
        self.sclSeek.handler_unblock_by_func(self.onSeekValueChanged)

    def onCurrentTrackMoved(self, hasPrevious, hasNext):
        """ Update previous and next buttons """
        self.btnNext.set_sensitive(hasNext)
        self.btnPrev.set_sensitive(hasPrevious)

    def onPaused(self):
        """ The playback has been paused """
        self.btnPlay.set_image(gtk.image_new_from_pixbuf(consts.playIcon))

    def onUnpaused(self):
        """ The playback has been unpaused """
        self.btnPlay.set_image(gtk.image_new_from_pixbuf(consts.pauseIcon))

    def onSeekValueChanged(self, range):
        """ The user has moved the seek slider """
        modules.postMsg(consts.MSG_CMD_SEEK,
            {'seconds': int(range.get_value())})
        #self.sclDragged = True

    def onSeekChangingValue(self, range, scroll, seconds):
        """ The user is moving the seek slider """
        if seconds > self.length: seconds = self.length
        #print 193, seconds
        self.lblTime.set_label(
            '%s/%s' % (sec2str(int(seconds)), sec2str(self.length)))

    def onSeekButtonPressed(self, range, event):
        """ Mouse button has been pressed over the seek slider """
        self.sclDragged = True #FIXME

        if event.button == 1:
            event.button = 2
            range.emit('button-press-event', event)
            return True

    def onSeekButtonReleased(self, range, event):
        """ Mouse button has been released over the seek slider """
        self.sclDragged = False#FIXME

        if event.button == 1:
            event.button = 2
            range.emit('button-release-event', event)
            self.btnPlay.grab_focus()
            return True

    def onVolumeChanged(self, button, value):
        """ The volume has been changed """        
        modules.postMsg(consts.MSG_CMD_SET_VOLUME, {'level': round(value, 1)})
        self.btnPlay.grab_focus()
