# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz #FIXME

import gobject, gtk, modules

from tools import consts, prefs
from gettext import gettext as _

MOD_INFO = ('Equalizer', _('Equalizer'), _('Tune the frequency of the bands'),
    False, False, 1)

(
    ROW_PRESET_IS_SEPARATOR,
    ROW_PRESET_NAME,
    ROW_PRESET_VALUES,
) = range(3)

class Equalizer(modules.Module):

    def __init__(self):
        """ Initialize the module """
        handlers = {
            consts.MSG_EVT_APP_STARTED:  self.onAppStarted,
            consts.MSG_EVT_MOD_LOADED:   self.onModLoaded,
            consts.MSG_EVT_MOD_UNLOADED: self.onModUnloaded,
                   }
        modules.Module.__init__(self, handlers)
        
    def onAppStarted(self):#FIXME
        """ The application has started """
        self.lvls      = prefs.get(__name__, 'levels', [0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
        self.preset    = prefs.get(__name__, 'preset', 'Flat')
        self.cfgWindow = None

        # connect menu item
        wtree = prefs.getWidgetsTree()
        menuitem = wtree.get_object('menu-eq')
        #menuitem.set_image(gtk.image_new_from_file(
            #'%s/decibel-eq.png' % consts.dirPix))
        menuitem.connect('activate', lambda w:
            self.configure(wtree.get_object('win-main')))
        menuitem.set_sensitive(True)

        # MSG_CMD_SET_EQZ_LVLS
        modules.postMsg(consts.MSG_CMD_SET_EQZ_LVLS, {'lvls': self.lvls})

    def onModLoaded(self):
        """ The module has been loaded """
        from gui import infoMsgBox

        gobject.idle_add(infoMsgBox, None, _('Restart required'),
            _('The application must be restarted.'))

        # MSG_CMD_ENABLE_EQZ True
        modules.postMsg(consts.MSG_CMD_ENABLE_EQZ, {'state': True})

    def onModUnloaded(self):
        """ The module has been unloaded """
        # MSG_CMD_ENABLE_EQZ False
        modules.postMsg(consts.MSG_CMD_ENABLE_EQZ, {'state': False})
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def configure(self, parent):
        """ Show the configuration dialog """
        if not self.cfgWindow :
            import gui.window

            self.cfgWindow = gui.window.Window('Equalizer.glade', 'vbox1',
                parent, MOD_INFO[1], 320, 1)
            self.combo      = self.cfgWindow.getWidget('combo-presets')
            self.comboStore = gtk.ListStore(
                gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
            self.newLvls = []
            self.timer   = None
            self.scales  = []
            # set scales
            for i in xrange(10):
                self.scales.append(self.cfgWindow.getWidget('vscale' + str(i)))
                self.scales[i].set_range(-24, 12)
                self.scales[i].set_value(self.lvls[i])
                self.scales[i].connect('value-changed', self.onScaleValueChanged, i)
            # set combo box
            txtRenderer = gtk.CellRendererText()
            self.combo.pack_start(txtRenderer, True)
            self.combo.add_attribute(txtRenderer, 'text', ROW_PRESET_NAME)
            self.combo.set_model(self.comboStore)
            self.combo.set_row_separator_func(lambda model, iter:
                model.get_value(iter, ROW_PRESET_IS_SEPARATOR))
            # add presets to combo box
            self.comboStore.append((False, 'Classic V',
                ( 7,  5,  0,  -5,  -8,  -7,  -4,  -1,   3,   5)))
            self.comboStore.append((False, 'Classical',
                ( 0,  0,  0,   0,   0,   0,   0,  -2,  -5,  -6)))
            self.comboStore.append((False, 'Dance',
                ( 6,  5,  4,   3,   1,   0,  -3,  -5,  -5,   0)))
            self.comboStore.append((False, 'Flat',
                ( 0,  0,  0,   0,   0,   0,   0,   0,   0,   0)))
            self.comboStore.append((False, 'Live',
                (-4, -2,  0,   2,   3,   3,   3,   3,   2,   0)))
            self.comboStore.append((False, 'Metal',
                ( 3,  4,  5,   1,  -2,   0,   1,   1,  -1,  -1)))
            self.comboStore.append((False, 'Pop',
                ( 3,  6,  3,  -2,  -4,  -3,   0,   2,   3,   5)))
            self.comboStore.append((False, 'Reggae',
                ( 1,  1,  1,   0,  -3,   0,   3,   4,   2,   1)))
            self.comboStore.append((False, 'Rock',
                ( 5,  4,  2,  -2,  -3,  -3,   2,   4,   5,   5)))
            self.comboStore.append((False, 'Techno',
                ( 4,  4,  3,   2,   0,  -4,  -2,   0,   3,   4)))
            # select current entry
            if self.preset is None:
                self.comboStore.insert(0, (False, '~', None))
                self.comboStore.insert(1, (True,  '',          None))
                self.combo.set_active(0)
            else:
                for i, preset in enumerate(self.comboStore):
                    if preset[ROW_PRESET_NAME] == self.preset:
                        self.combo.set_active(i)
                        break
            # cfg events
            self.cfgWindow.getWidget('btn-save').connect('clicked',
                self.onBtnSave)
            self.cfgWindow.getWidget('btn-open').connect('clicked',
                self.onBtnOpen)
            self.cfgWindow.getWidget('btn-close').connect('clicked', lambda btn:
                self.cfgWindow.hide())
            self.combo.connect('changed', self.onPresetChanged)
        # cfg show()
        if not self.cfgWindow.isVisible():
            self.cfgWindow.getWidget('btn-close').grab_focus()
        self.cfgWindow.show()

    def onPresetChanged(self, combo):
        """ A preset has been selected """
        idx = combo.get_active()
        if idx != -1:
            iter = self.comboStore.get_iter(idx)
            preset = self.comboStore.get_value(iter, ROW_PRESET_NAME)
            self.jumpTonewLvls(self.comboStore.get_value(iter, ROW_PRESET_VALUES))
            # remove '~' entry
            if not self.preset:
                self.comboStore.remove(self.comboStore.get_iter((0, )))
                self.comboStore.remove(self.comboStore.get_iter((0, )))

            self.preset = preset
            prefs.set(__name__, 'preset', self.preset)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onBtnOpen(self, btn):
        """ Load the levels from a file"""
        from gui import fileChooser, errorMsgBox

        inFile = fileChooser.openFile(self.cfgWindow, _('Load preset...'))
        if inFile is not None:
            input = open(inFile, 'rt')
            lines = input.readlines()
            input.close()
            lvls      = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
            isInvalid = True
            if len(lines) == 10:
                isInvalid = False
                for i in xrange(10):
                    elts = lines[i].split()
                    try:
                        if len(elts) == 1:
                            lvls[i] = float(elts[0])
                            if lvls[i] >= -24 and lvls[i] <= 12:
                                continue
                    except:
                        pass
                    isInvalid = True
                    break
            if isInvalid:
                errorMsgBox(self.cfgWindow, _('File not loadable'),
                    _('The format of the file is incorrect.'))
            else:
                self.jumpTonewLvls(lvls)
                # rename preset to '~'
                if self.preset:
                    self.preset = None
                    prefs.set(__name__, 'preset', self.preset)
                    self.combo.handler_block_by_func(self.onPresetChanged)
                    self.comboStore.insert(0, (False, '~', None))
                    self.comboStore.insert(1, (True,  '',          None))
                    self.combo.set_active(0)
                    self.combo.handler_unblock_by_func(self.onPresetChanged)

    def onBtnSave(self, btn):
        """ Save the current levels to a file"""
        from gui import fileChooser

        outFile = fileChooser.save(self.cfgWindow, _('Save preset...'), 'levels.dat')
        if outFile:
            output = open(outFile, 'wt')
            for i in xrange(10):
                output.write(str(self.lvls[i]) + '\n')
            output.close()
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onScaleValueChanged(self, scale, idx):
        """ The user has moved one of the scales, rename preset to '~' """
        if self.preset:
            self.preset = None
            prefs.set(__name__, 'preset', self.preset)
            self.combo.handler_block_by_func(self.onPresetChanged)
            self.comboStore.insert(0, (False, '~', None))
            self.comboStore.insert(1, (True,  '',          None))
            self.combo.set_active(0)
            self.combo.handler_unblock_by_func(self.onPresetChanged)
        self.lvls[idx] = scale.get_value()
        prefs.set(__name__, 'levels', self.lvls)
        # MSG_CMD_SET_EQZ_LVLS
        modules.postMsg(consts.MSG_CMD_SET_EQZ_LVLS, {'lvls': self.lvls})

    def jumpTonewLvls(self, newLvls):
        """ Move the scales until they reach some target levels """
        if self.timer is not None:
            gobject.source_remove(self.timer)
        self.timer      = gobject.timeout_add(20, self.timerFunc)
        self.newLvls = newLvls
        for i in xrange(10):
            self.scales[i].handler_block_by_func(self.onScaleValueChanged)

    def timerFunc(self):
        """ Move a bit the scales to their target value """
        isFinished = True
        # move the scales a bit
        for i in xrange(10):
            currLvl    = self.scales[i].get_value()
            targetLvl  = self.newLvls[i]
            difference = targetLvl - currLvl
            if abs(difference) <= 0.25:
                newLvl = targetLvl
            else:
                newLvl     = currLvl + (difference / 8.0)
                isFinished = False
            self.lvls[i] = newLvl
            self.scales[i].set_value(newLvl)
        # MSG_CMD_SET_EQZ_LVLS
        modules.postMsg(consts.MSG_CMD_SET_EQZ_LVLS, {'lvls': self.lvls})
        if isFinished:
            self.timer = None
            prefs.set(__name__, 'levels', self.lvls)
            # make sure labels are up to date and unblock handlers
            for i in xrange(10):
                self.scales[i].queue_draw()
                self.scales[i].handler_unblock_by_func(self.onScaleValueChanged)
            return False
        return True
