# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz #FIXME

import gtk, media, modules, os, tools

from gobject import idle_add
from os.path import isdir, isfile
from tools import consts, prefs, htmlEscape

MOD_INFO = ('Explorer', 'Explorer', '', True, False, None)
MOD_NAME = MOD_INFO[0]
HIDDEN_FILES = False

(
    ROW_PIXBUF,
    ROW_NAME,
    ROW_TYPE,
    ROW_FULLPATH
) = range(4)

(
    TYPE_DIR,
    TYPE_FILE,
    TYPE_NONE
) = range(3)

class Explorer(modules.Module):

    def __init__(self):
        """ Constructor """
        handlers = {
            consts.MSG_EVT_APP_QUIT:         self.onAppQuit,
            consts.MSG_EVT_APP_STARTED:      self.onAppStarted,
                   }
        modules.Module.__init__(self, handlers)

    def onAppStarted(self):#FIXME
        """ The module has been loaded """
        self.cfgWin     = None
        self.rootDir    = consts.dirBaseUsr
        self.hiddenFs   = prefs.get(__name__, 'hidden-files', HIDDEN_FILES)

        self.tree       = None
        self.treeState  = {}

        #self.scrolled = gtk.ScrolledWindow()
        #self.scrolled.set_shadow_type(gtk.SHADOW_IN)
        #self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        self.notebook = prefs.getWidgetsTree().get_object('notebook2')
        self.notebook.connect('switch-page', self.onNewExplorer)


        self.onNewExplorer(None, None, 0)
        self.notebook.set_current_page(0)

    def onAppQuit(self):
        """ The module is going to be unloaded """
        #prefs.set(__name__, 'hidden-files', self.hiddenFs)

    def onNewExplorer(self, widget, pages, page_num):
        """ A new page has been selected """
        if page_num == 0:
            if not self.tree:
                self.tree = self.__createTree()

                self.exploreDir(None, self.rootDir)
                if len(self.tree) != 0:
                    self.tree.scroll_to_cell(0)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __createTree(self):
        """ Create the tree used to display the file system """
        from gobject import TYPE_STRING, TYPE_INT
        from gui import extTreeview

        columns = (('', [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf),
            (gtk.CellRendererText(), TYPE_STRING)], True),
            (None, [(None, TYPE_INT)], False),
            (None, [(None, TYPE_STRING)], False))
        tree = extTreeview.ExtTreeView(columns, True)
        tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_TRACKS]])
        tree.connect('drag-data-get', self.onDragDataGet)
        tree.connect('key-press-event', self.onKeyPressed)
        tree.connect('exttreeview-button-pressed', self.onMouseButton)
        tree.connect('exttreeview-row-collapsed', self.onRowCollapsed)
        tree.connect('exttreeview-row-expanded', self.onRowExpanded)

        prefs.getWidgetsTree().get_object('scrolled0').add(tree)

        return tree

    def refreshTree(self, treePath=None):
        """ Refresh the tree, starting from treePath """
        if not treePath:
            directory = self.rootDir
        else:
            directory = self.tree.getItem(treePath, ROW_FULLPATH)

        directories, playlists, mediaFiles = self.getDirContents(directory)

        disk                   = directories + playlists + mediaFiles
        diskIndex              = 0
        childIndex             = 0
        childLeftIntentionally = False

        while diskIndex < len(disk):
            rowPath = self.tree.getChild(treePath, childIndex)

            # Did we reach the end of the tree?
            if rowPath is None:
                break

            file      = disk[diskIndex]
            cmpResult = cmp(self.sortKey(self.tree.getRow(rowPath)),
                self.sortKey(file))

            if cmpResult < 0:
                # We can't remove the only child left, to prevent the node from
                # being closed automatically
                if self.tree.getNbChildren(treePath) == 1:
                    childLeftIntentionally = True
                    break

                self.tree.removeRow(rowPath)
            else:
                if cmpResult > 0:
                    self.tree.insertRowBefore(file, treePath, rowPath)
                diskIndex  += 1
                childIndex += 1

        # If there are tree rows left, all the files are no longer there
        if not childLeftIntentionally:
            while childIndex < self.tree.getNbChildren(treePath):
                self.tree.removeRow(self.tree.getChild(treePath, childIndex))

        # Disk files left?
        while diskIndex < len(disk):
            self.tree.appendRow(disk[diskIndex], treePath)
            diskIndex += 1

        # Deprecated child left? (must be done after addition)
        if childLeftIntentionally:
            self.tree.removeRow(self.tree.getChild(treePath, 0))

        # Update nodes' appearance
        if len(directories) != 0:
            idle_add(self.updateDirNodes(treePath).next)

        # Recursively refresh expanded rows
        for child in self.tree.iterChildren(treePath):
            if self.tree.row_expanded(child):
                idle_add(self.refreshTree, child)


    def restoreTreeDump(self, dump, parent=None):#FIXME
        """ Recursively restore the dump under the given parent,
            none for the root of the tree
        """
        for item in dump:
            (name, type, path) = item[0]

            if type == TYPE_FILE:
                self.tree.appendRow(
                    (consts.dirIcon, name, TYPE_FILE, path), parent)
            else:
                newNode = self.tree.appendRow(
                    (consts.dirIcon, name, TYPE_DIR, path), parent)

                if item[1] is not None:
                    fakeChild = self.tree.appendRow(
                        (consts.dirIcon, '', TYPE_NONE, ''), newNode)

                    if len(item[1]) != 0:
                        # Expand the row before adding the real children when
                        # there is at least one child
                        self.tree.expandRow(newNode)
                        self.restoreTreeDump(item[1], newNode)
                        self.tree.removeRow(fakeChild)


    def sortKey(self, row):
        """ Key function used to compare two rows of the tree """
        return row[ROW_NAME].lower()



#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def getDirContents(self, directory):
        """ Return a tuple of sorted rows (dirs, playlists, files) """
        playlists   = []
        mediaFiles  = []
        directories = []

        for (file, path) in tools.listDir(directory, self.hiddenFs):

            if isdir(path):
                directories.append((consts.dirIcon,
                    htmlEscape(unicode(file, errors='replace')),
                    TYPE_DIR, path))

            elif isfile(path):
                if media.isSupported(file):
                    mediaFiles.append((consts.fileIcon,
                        htmlEscape(unicode(file, errors='replace')),
                        TYPE_FILE, path))

                elif media.isSupportedPlaylist(file):
                    playlists.append((consts.playIcon,
                        htmlEscape(unicode(file, errors='replace')),
                        TYPE_FILE, path))

        playlists.sort(key=self.sortKey)
        mediaFiles.sort(key=self.sortKey)
        directories.sort(key=self.sortKey)

        return (directories, playlists, mediaFiles)

    def exploreDir(self, parent, directory, fakeChild=None):
        """ List the contents of the given directory and append it to the tree
            as a child of parent. If fakeChild is not None, remove it when the
            real contents has been loaded.
        """
        directories, playlists, mediaFiles = self.getDirContents(directory)

        self.tree.appendRows(directories, parent)
        self.tree.appendRows(playlists,   parent)
        self.tree.appendRows(mediaFiles,  parent)

        if fakeChild:
            self.tree.removeRow(fakeChild)

        idle_add(self.updateDirNodes(parent).next)

    def updateDirNodes(self, parent):
        """ This generator updates the directory nodes """
        for child in self.tree.iterChildren(parent):
            if self.tree.getItem(child, ROW_TYPE) != TYPE_DIR:
                break

            directory  = self.tree.getItem(child, ROW_FULLPATH)
            hasContent = False
            if os.access(directory, os.R_OK | os.X_OK):
                for (file, path) in tools.listDir(directory, self.hiddenFs):
                    if isdir(path) or (isfile(path) \
                        and (media.isSupported(file) \
                        or media.isSupportedPlaylist(file))):
                        hasContent = True
                        break

            if hasContent and self.tree.getNbChildren(child) == 0:
                self.tree.appendRow(
                    (consts.dirIcon, '', TYPE_NONE, ''), child)
            elif not hasContent and self.tree.getNbChildren(child) > 0:
                self.tree.removeAllChildren(child)

            yield True

        yield False
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onAddFolder(self, path):
        """ Add a new folder to index """
        modules.postMsg(consts.MSG_CMD_INDEX_PATH, {'path': path})

    def onRowExpanded(self, tree, path):
        """ Replace the fake child by the real children """
        idle_add(self.exploreDir, path, tree.getItem(path, ROW_FULLPATH),
            tree.getChild(path, 0))

    def onRowCollapsed(self, tree, path):
        """ Replace all children by a fake child """
        tree.removeAllChildren(path)
        tree.appendRow((consts.dirIcon, '', TYPE_NONE, ''), path)

    def onDragDataGet(self, tree, context, selection, info, time):
        """ Provide information about the data being dragged """
        allTracks = media.getTracks([row[ROW_FULLPATH]
            for row in self.tree.getSelectedRows()], not self.hiddenFs)

        selection.set(consts.DND_TARGETS[consts.DND_DAP_TRACKS][0], 8,
            '\n'.join([track.serialize()
            for track in allTracks]))

    def onMouseButton(self, tree, event, path):
        """ A mouse button has been pressed """
        if event.button == 3:
            self.onShowPopupMenu(tree, event.button, event.time, path)
        elif path is not None:
            if event.button == 2:
                self.onSetTracklist(False, path)
            elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
                if   tree.getItem(path, ROW_PIXBUF) != consts.dirIcon:
                    self.onSetTracklist(True)
                elif tree.row_expanded(path):
                    tree.collapse_row(path)
                else:
                    tree.expand_row(path, False)

    def onSetTracklist(self, replace, path=None):
        """ Extend/Replace the tracklist, if 'None' get the current selection """
        if path is None:
            tracks = media.getTracks([row[ROW_FULLPATH]
                for row in self.tree.getSelectedRows()], not self.hiddenFs)
        else:
            tracks = media.getTracks([self.tree.getRow(path)[ROW_FULLPATH]],
                not self.hiddenFs)

        if replace:
            modules.postMsg(consts.MSG_CMD_TRACKLIST_SET,
                {'tracks': tracks, 'playNow': True})
        else:
            modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD,
                {'tracks': tracks, 'playNow': False})
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onShowPopupMenu(self, tree, button, time, path):
        """ Show a popup menu """
        popup = gtk.Menu()

        play = gtk.ImageMenuItem(gtk.STOCK_MEDIA_PLAY)
        popup.append(play)
        if path is None: play.set_sensitive(False)
        else: play.connect('activate', lambda w: self.onSetTracklist(True))

        add = gtk.ImageMenuItem(gtk.STOCK_ADD)
        popup.append(add)
        if path is None: add.set_sensitive(False)
        else: add.connect('activate', lambda w: self.onSetTracklist(False))

        popup.append(gtk.SeparatorMenuItem())

        refresh = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
        refresh.connect('activate', lambda w: self.refreshTree())
        popup.append(refresh)

        find = gtk.ImageMenuItem(gtk.STOCK_FIND)#FIXME
        try:
            dirPath = self.tree.getRow(path)[ROW_FULLPATH]
            if not os.path.isdir(dirPath): raise
        except:
            dirPath = None
        find.set_sensitive(dirPath != None)
        find.connect('activate', lambda w: self.onAddFolder(dirPath))
        popup.append(find)

        popup.append(gtk.SeparatorMenuItem())

        collapse = gtk.ImageMenuItem(gtk.STOCK_CLOSE)
        collapse.connect('activate', lambda w: tree.collapse_all())
        collapse.set_sensitive(False)
        for child in self.tree.iterChildren(None):
            if self.tree.row_expanded(child):
                collapse.set_sensitive(True)
                break
        popup.append(collapse)

        popup.show_all()
        popup.popup(None, None, None, button, time)

    def onKeyPressed(self, tree, event):
        """ A key has been pressed """
        keyname = gtk.gdk.keyval_name(event.keyval)

        if keyname == 'F5':       self.refreshTree()
        elif keyname == 'plus':   tree.expandRows()
        elif keyname == 'Left':   tree.collapseRows()
        elif keyname == 'Right':  tree.expandRows()
        elif keyname == 'minus':  tree.collapseRows()
        elif keyname == 'space':  tree.switchRows()
        elif keyname == 'Return': self.onSetTracklist(True)
