# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz #FIXME

import gtk, modules, os, tools
from tools import consts, prefs, htmlEscape, log, pickleLoad, pickleSave
from gobject import idle_add


MOD_INFO = ('Finder', 'Finder', '', True, False, None)
MOD_NAME = MOD_INFO[0]
LIB_DIR = os.path.join(consts.dirCfg, MOD_NAME)
LIB_INDEX = 'index.ini'

(
    TYPE_ARTIST,
    TYPE_ALBUM,
    TYPE_TRACK,
    TYPE_HEADER,
    TYPE_NONE
) = range(5)

(
    ROW_PIXBUF,
    ROW_NAME,
    ROW_TYPE,
    ROW_PATH,
    ROW_DATA
) = range(5)

FAKECHILD = (None, '', TYPE_NONE, '', None)

class Finder(modules.Module):

    def __init__(self):
        """ Initialization """
        handlers = {
            consts.MSG_CMD_INDEX_PATH:  self.onNewFolder,
            consts.MSG_EVT_APP_STARTED: self.onAppStarted,
            #onsts.MSG_EVT_APP_QUIT:    self.onAppQuit,
            }
        modules.Module.__init__(self, handlers)

    def onAppStarted(self):#FIXME
        """ The module has been loaded """
        self.lastSearch = None
        self.libPath = prefs.get(__name__, 'lib-path', None)

        self.tree = self.__createTree()

        #self.scrolled = gtk.ScrolledWindow()
        #self.scrolled.set_shadow_type(gtk.SHADOW_IN)
        #self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        #self.scrolled = prefs.getWidgetsTree().get_object('scrolledwin2')
        #self.scrolled.add(self.tree)
        #self.scrolled.show()

        #self.entry = gtk.Entry()
        self.entry = prefs.getWidgetsTree().get_object('txt-entry')
        self.entry.connect('activate', self.onSearch)
        self.entry.connect('icon-press', self.__delSearch)
        self.entry.connect('focus-out-event', lambda w,e: w.set_text(''))

        #self.progress = gtk.ProgressBar()
        self.progress = prefs.getWidgetsTree().get_object('progress-bar')

        #self.vbox = gtk.VBox()
        #self.vbox.pack_start(self.entry, False, False, 0)
        #self.vbox.add(self.scrolled)
        #self.vbox.set_spacing(6)
        #self.vbox.show_all()

        self.notebook = prefs.getWidgetsTree().get_object('notebook2')


        try:
            libIndex = pickleLoad(os.path.join(LIB_DIR, LIB_INDEX)) 
        except Exception, err:
            log.c.warning(err)

            if not os.path.exists(LIB_DIR): os.mkdir(LIB_DIR)
            self.libPath = None
            #error = 'Index not found, choice a folder from explorer.'
            #self.tree.replaceContent(
            #    [(consts.errIcon, error, TYPE_NONE, None, None)])

        self.entry.set_sensitive(self.libPath != None)

    def onAppQuit(self):
        """ The module has been unloaded """
        #pass

    def onSearch(self, widget):
        """ Search for an artist in the current library """
        currentSearch = widget.get_text()
        if currentSearch == self.lastSearch: return
        #chars = len(currentSearch)
        #if chars <= 2:
        #    if chars == 0:
        #        #widget.set_icon_from_stock(1, None)
        #        currentSearch = None
        #    else:
        #        widget.set_text('')
        #        return
            
        self.lastSearch = currentSearch

        self.loadArtists(self.tree)



    def __delSearch(self, widget, icon, event):
        """ Called at icon pressed on search entry """
        if icon == gtk.ENTRY_ICON_SECONDARY:
            widget.set_text('')
            widget.set_icon_from_stock(1, None)#FIXME
            #widget.activate()

        self.notebook.set_current_page(0)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __createTree(self):
        """ Create the main tree, add it to the scrolled window """
        from gui.extTreeview import ExtTreeView
        from gobject import TYPE_STRING, TYPE_INT, TYPE_PYOBJECT

        txtRdr         = gtk.CellRendererText()
        pixbufRdr      = gtk.CellRendererPixbuf()

        columns = (
            ('',    [(pixbufRdr, gtk.gdk.Pixbuf), (txtRdr, TYPE_STRING)], False),
            (None,  [(None, TYPE_INT)], False),
            (None,  [(None, TYPE_STRING)], False),
            (None,  [(None, TYPE_PYOBJECT)], False))

        tree = ExtTreeView(columns, True)
        tree.set_search_equal_func(self.__searchFunc)
        tree.get_column(0).set_cell_data_func(txtRdr, self.__drawCell)
        tree.get_column(0).set_cell_data_func(pixbufRdr, self.__drawCell)
        tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_TRACKS]])

        tree.connect('drag-data-get',              self.onDragData)
        tree.connect('key-press-event',            self.onKeyPressed)
        tree.connect('exttreeview-row-expanded',   self.onRowExpanded)
        tree.connect('exttreeview-row-collapsed',  self.onRowCollapsed)
        tree.connect('exttreeview-button-pressed', self.onButtonPressed)

        prefs.getWidgetsTree().get_object('scrolled1').add(tree)

        return tree

    def __searchFunc(self, model, column, key, iter):
        """ Check whether the given key matches the current candidate (iter) """
        return model.get_value(iter, ROW_NAME)[:len(key)].lower() != key.lower()

    def __drawCell(self, column, cell, model, iter):
        """ Use a different background color for alphabetical headers """
        if model.get_value(iter, ROW_TYPE) == TYPE_HEADER:
            cell.set_property('cell-background-gdk',
                self.tree.style.base[gtk.STATE_PRELIGHT])
        else:
            cell.set_property('cell-background', None)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def refreshLibrary(self, libPath):#FIXME
        """ Refresh the given library, must be called through idle_add() """
        import collections, shutil, media
        from media.filetrack import FileTrack
        from os.path import isdir, isfile

        # libIndex = {artists:{albums:[tracks,],},}
        libIndex = {}
        # libFile for libNew saving
        libStr   = os.path.basename(libPath).lower()
        libName  = '%s.ini' % abs(hash(libStr))
        libFile  = os.path.join(LIB_DIR, libName)
        # libNew = {dirs:(dirtime,subdir,{filename:[time,instance,fullpath]}),}
        libNew   = {} 
        libOld   = pickleLoad(libFile) if (os.path.exists(libFile)) else {}
        # libDeque for directories walking
        libDeque = collections.deque((libPath,))

        self.progress.show()
        self.entry.hide()

        yield True

        if not os.path.exists(libPath):
            libDeque.pop()

        while len(libDeque) != 0:
            currDir      = libDeque.pop()
            currDirMTime = os.stat(currDir).st_mtime

            if currDir in libOld:
                oldDirMTime, oldDirectories, oldFiles = libOld[currDir]
            else:
                oldDirMTime, oldDirectories, oldFiles = -1, [], {}

            if currDirMTime == oldDirMTime:
                files, directories = oldFiles, oldDirectories
            else:
                files, directories = {}, []
                for (filename, fullPath) in tools.listDir(currDir):
                    if isdir(fullPath):
                        directories.append(fullPath)
                    elif isfile(fullPath) and media.isSupported(filename):
                        if filename in oldFiles:
                            files[filename] = oldFiles[filename]
                        else:
                            files[filename] = [-1, FileTrack(fullPath), fullPath]

            self.progress.pulse()

            yield True

            for filename, (oldMTime, track, fullPath) in files.iteritems():
                mTime = os.stat(fullPath).st_mtime
                if mTime != oldMTime:
                    track = media.getTrackFromFile(fullPath)
                    files[filename] = [mTime, track, fullPath]#FIXME
                    media.setTrackToIndex(track, fullPath)#FIXME

                artist, album = track.getArtist(), track.getAlbum()
                if artist.startswith(' '): artist = '~'
                # build libIndex
                if artist in libIndex:
                    albumsIndex = libIndex[artist]
                    try:
                        albumsIndex[album].append(track)
                    except:
                        albumsIndex[album] = [track,]
                else:
                    libIndex[artist] = {album: [track,]}

                self.progress.pulse()

            libNew[currDir] = (currDirMTime, directories, files)
            libDeque.extend(directories)

            self.progress.pulse()

            yield True

        # stored on libName dir, refreshed at loading previous folders
        pickleSave(libFile, libNew)
        # stored on root dir, replaced at loading new folder
        pickleSave(os.path.join(LIB_DIR, LIB_INDEX), libIndex)

        self.loadArtists(self.tree)
        
        self.progress.hide()
        self.entry.set_sensitive(True)
        self.entry.show()
        self.entry.activate()
        self.entry.set_icon_from_stock(1, gtk.STOCK_CLEAR)
     
        #print "Statistics: %s artists, %s albums, %s tracks" % (
        #log.c.info("Statistics: %s tracks, %s artists" % (
        #    len(libNew),
        #    len(libIndex)))
        #sum([albums for artists, index, albums in artistsIndex]),

        yield False
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def loadArtists(self, tree):
        """ Initial load of the artists of the given INDEX """
        libIndex = pickleLoad(os.path.join(LIB_DIR, LIB_INDEX))
        rows = []
        prevChar = '~'

        for artist in sorted(libIndex.keys()):

            if self.lastSearch:
                cmp1, cmp2 = artist.lower(), self.lastSearch.lower()#FIXME
                if len(cmp2) <= 2 and not cmp1.startswith(cmp2):
                    continue
                if len(cmp2) >= 3 and not cmp2 in cmp1:
                    continue

            if len(artist) != 0:
                currChar = unicode(artist, errors='replace')[0].lower()
            else:
                currChar = prevChar

            if prevChar != currChar and \
                not (prevChar.isdigit() and currChar.isdigit()):
                prevChar = currChar

                if currChar.isdigit():
                    rows.append((None, '<b>0 - 9</b>',
                        TYPE_HEADER, None, None))
                else:
                    rows.append((None, '<b>%s</b>' % currChar.upper(),
                        TYPE_HEADER, None, None))

            rows.append((consts.dirIcon, htmlEscape(artist),
                TYPE_ARTIST, artist, libIndex[artist]))

        if self.lastSearch and len(rows) == 0:#FIXME
            self.entry.set_text('')
            return
        else:
            self.entry.set_icon_from_stock(1, gtk.STOCK_CLEAR)
            self.notebook.set_current_page(1)


        tree.replaceContent(rows)

        for node in tree.iterChildren(None):
            if tree.getItem(node, ROW_TYPE) == TYPE_ARTIST:
                tree.appendRow(FAKECHILD, node)


    def loadAlbums(self, tree, node, fakeChild):
        """ Initial load of the albums of the given node TYPE_ARTIST """
        rows      = []
        #artist   = tree.getItem(node, ROW_PATH)
        albums    = tree.getItem(node, ROW_DATA)
 
        for album in sorted(albums.keys()):
            rows.append((consts.dirIcon, htmlEscape(album),
                TYPE_ALBUM, album, albums[album]))

        tree.appendRows(rows, node)
        tree.removeRow(fakeChild)
        for child in tree.iterChildren(node):
            tree.appendRow(FAKECHILD, child)


    def loadTracks(self, tree, node, fakeChild):
        """ Initial load of all tracks of the given node TYPE_ALBUM """
        #album    = tree.getItem(node, ROW_PATH)
        tracks    = tree.getItem(node, ROW_DATA)

        tracks = sorted([track for track in tracks], lambda t1, t2: cmp(
        t1.getTitle(), t2.getTitle()
        ))

        rows = [(consts.fileIcon, htmlEscape(track.getTitle()),
            TYPE_TRACK, track.getFilePath(), track) for track in tracks]

        tree.appendRows(rows, node)
        tree.removeRow(fakeChild)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __getTracksFromPaths(self, tree, paths):#FIXME
        """ Return the list of tracks extracted from:
            * The list 'paths' if it is not None
            * The currently selected rows if 'paths' is None
        """
        from sys import maxint

        headers = []
        tracks = []

        # get headers, removed below from selected rows
        for path in tree.iterChildren(None):
            if tree.getItem(path, ROW_TYPE) == TYPE_HEADER:
                headers.append(path)       
        # get selected rows
        if not paths:
            paths = [path for path in tree.getSelectedPaths()
                if path not in headers]
        # get tracks from row data
        for currPath in paths:
            row = tree.getRow(currPath)
            if row[ROW_TYPE] == TYPE_TRACK:
                tracks.append(row[ROW_DATA])
            elif row[ROW_TYPE] == TYPE_ALBUM:
                tracks.extend(row[ROW_DATA])
            elif row[ROW_TYPE] == TYPE_ARTIST:
                #print 360,row[ROW_DATA]
                for album in row[ROW_DATA].keys():
                    tracks.extend(row[ROW_DATA][album])
            elif row[ROW_TYPE] == TYPE_HEADER:
                pass

        return tracks

    def playPaths(self, tree, paths, replace):#FIXME
        """ Replace/extend the tracklist, if the list 'paths' is None,
            use the current selection.
        """
        tracks = self.__getTracksFromPaths(tree, paths)

        if replace: modules.postMsg(
            consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
        else: modules.postMsg(
            consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks, 'playNow': False})
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onAddFolder(self):#FIXME
        """ Add a new folder to the list """
        from gui import fileChooser
        from gettext import ngettext, gettext as _

        libPath = fileChooser.openDirectory(None, _('Open folder...'))
        if libPath:
            idle_add(self.refreshLibrary(libPath).next)
            prefs.set(__name__, 'lib-path', libPath)

            if os.path.basename(libPath) == os.path.expanduser('~'):
                libName = 'Home'#FIXME
            else:
                libName = os.path.basename(libPath)

            self.libPath = libPath

    def onNewFolder(self, path):
        """ Add a new folder to index """
        idle_add(self.refreshLibrary(path).next)
        prefs.set(__name__, 'lib-path', path)

        self.libPath = path

    def onDragData(self, tree, context, selection, info, time):
        """ Provide information about the data being dragged """
        serializedTracks = '\n'.join(
            [track.serialize() \
            for track in self.__getTracksFromPaths(tree, None)])
        selection.set(
            consts.DND_TARGETS[consts.DND_DAP_TRACKS][0], 8, serializedTracks)

    def onRowExpanded(self, tree, node):
        """ Populate the expanded row """
        if tree.getItem(node, ROW_TYPE) == TYPE_ARTIST:
            self.loadAlbums(tree, node, tree.getChild(node, 0))
        else:
            self.loadTracks(tree, node, tree.getChild(node, 0))

    def onRowCollapsed(self, tree, node):
        """ Replace all children of the node by a fake child """
        tree.removeAllChildren(node)
        tree.appendRow(FAKECHILD, node)

    def onButtonPressed(self, tree, event, path):
        """ A mouse button has been pressed """
        if event.button == 3:
            self.onPopupMenu(tree, event.button, event.time, path)

        elif path is not None and tree.getItem(path, ROW_TYPE) != TYPE_NONE:
            if event.button == 2:
                self.playPaths(tree, [path], False)
            elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:

                if tree.getItem(path, ROW_PIXBUF) != consts.dirIcon:
                    self.playPaths(tree, None, True)
                elif tree.row_expanded(path):
                    tree.collapse_row(path)
                else:
                    tree.expand_row(path, False)

    def onPopupMenu(self, tree, button, time, path):
        """ Show a popup menu """
        popup = gtk.Menu()

        play = gtk.ImageMenuItem(gtk.STOCK_MEDIA_PLAY)
        popup.append(play)
        if path is None: play.set_sensitive(False)
        else: play.connect('activate', lambda w:
            self.playPaths(tree, None, True))

        add = gtk.ImageMenuItem(gtk.STOCK_ADD)
        popup.append(add)
        if path is None: add.set_sensitive(False)
        else: add.connect('activate', lambda w:
            self.playPaths(tree, None, False))

        popup.append(gtk.SeparatorMenuItem())

        refresh = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
        refresh.connect('activate', lambda w:
            idle_add(self.refreshLibrary(self.libPath).next))
        refresh.set_sensitive(self.libPath != None)
        popup.append(refresh)

        edit = gtk.ImageMenuItem(gtk.STOCK_EDIT)#FIXME
        edit.set_sensitive(False)
        #edit.set_image(gtk.image_new_from_stock(
        #    gtk.STOCK_HARDDISK, gtk.ICON_SIZE_MENU))
        #edit.connect('activate', lambda w: self.onAddFolder())
        popup.append(edit)

        popup.append(gtk.SeparatorMenuItem())

        collapse = gtk.ImageMenuItem(gtk.STOCK_CLOSE)
        collapse.connect('activate', lambda w: tree.collapse_all())
        collapse.set_sensitive(False)
        for child in tree.iterChildren(None):
            if tree.row_expanded(child):
                collapse.set_sensitive(True)
                break
        popup.append(collapse)

        popup.show_all()
        popup.popup(None, None, None, button, time)

    def onKeyPressed(self, tree, event):
        """ A key has been pressed """
        keyname = gtk.gdk.keyval_name(event.keyval)

        if keyname == 'F5' and self.libPath:
            idle_add(self.refreshLibrary(self.libPath).next)
        elif keyname == 'plus':   tree.expandRows()
        elif keyname == 'Left':   tree.collapseRows()
        elif keyname == 'Right':  tree.expandRows()
        elif keyname == 'minus':  tree.collapseRows()
        elif keyname == 'space':  tree.switchRows()
        elif keyname == 'Return': self.playPaths(tree, None, True)
