# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz #FIXME

import gtk, gst, gobject, modules, os.path

from media import gstrack
from tools import consts, prefs, log
from time import time

MOD_INFO = ('GSTPlayer', 'GSTPlayer', '', True, False, None)

class GSTPlayer(modules.Module):

    def __init__(self):
        """ Constructor """
        self.queueseek  = None
        self.seeked     = None
        self.timer      = None
        self.timeout    = None
        # init player before init any handler
        self.player = self.__init_player()

        handlers = {
            consts.MSG_CMD_ENABLE_EQZ:   self.onEqualizerLoaded,
            consts.MSG_CMD_SET_EQZ_LVLS: self.setEqualizerLvls,
            consts.MSG_CMD_SET_VOLUME:   self.onVolumeChanged,
            consts.MSG_CMD_STOP:         self.onStop,
            consts.MSG_CMD_PLAY:         self.onPlay,   
            consts.MSG_CMD_TOGGLE_PAUSE: self.onTogglePause,
            consts.MSG_CMD_SEEK:         self.onSeek,
                   }
        modules.Module.__init__(self, handlers)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __init_player(self):
        """ Create the GStreamer pipeline """
        # make gst pipeline
        player = gst.element_factory_make('playbin2', 'player')
        player.connect('about-to-finish', self.__onAboutToFinish)
        # make audioskink
        audiobin  = gst.Bin('audiobin')
        audiosink = gst.element_factory_make('autoaudiosink', 'audiosink')
        audiobin.add(audiosink)
        audiobin.add_pad(gst.GhostPad('sink', audiosink.get_pad('sink')))
        player.set_property('audio-sink', audiobin)
        # make fake videosink, mp4s fix
        videosink = gst.element_factory_make("fakesink")
        videosink.set_property("sync", True)
        player.set_property("video-sink", videosink)
        # get bus messages
        bus = player.get_bus()
        bus.add_signal_watch()
        bus.connect('message', self.__onGstMessage)
        # link equalizer
        if prefs.get(__name__, 'link-eq', False):
            self.equalizer = gst.element_factory_make('equalizer-10bands', 'eq')
            audiobin.add(self.equalizer)
            audiobin.get_pad('sink').set_target(self.equalizer.get_pad('sink'))
            self.equalizer.link(audiosink)
            # link replaygain
            replaygain = gst.element_factory_make('rgvolume', 'replaygain')
            audiobin.add(replaygain)
            audiobin.get_pad('sink').set_target(replaygain.get_pad('sink'))
            replaygain.link(self.equalizer)
            #print 72
        # done!
        return player
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onEqualizerLoaded(self, state):
        """Enable/disable the equalizer of 10 bands """
        prefs.set(__name__, 'link-eq', state)

    def setEqualizerLvls(self, lvls):
        """ Set the 10-bands of the equalizer (-24/+12) """
        if self.equalizer and len(lvls) == 10:
            self.equalizer.set_property('band0', lvls[0])
            self.equalizer.set_property('band1', lvls[1])
            self.equalizer.set_property('band2', lvls[2])
            self.equalizer.set_property('band3', lvls[3])
            self.equalizer.set_property('band4', lvls[4])
            self.equalizer.set_property('band5', lvls[5])
            self.equalizer.set_property('band6', lvls[6])
            self.equalizer.set_property('band7', lvls[7])
            self.equalizer.set_property('band8', lvls[8])
            self.equalizer.set_property('band9', lvls[9])
        else:
            self.lvls = lvls
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __onAboutToFinish(self, isLast):
        """ End of the track, set pos to -1 to avoid any next call """
        if self.position > 0:
            self.position = 0
            # MSG_EVT_TRACK_ENDED_OK
            modules.postMsg(consts.MSG_EVT_TRACK_ENDED_OK)           
        log.c.debug('<gst.Message about-to-finish>')
        #return True

    def __onGstMessage(self, bus, msg):
        """ A new message generated by the player """
        if msg.type == gst.MESSAGE_EOS:
            if self.position > 0:
                self.position = 0
                # MSG_EVT_TRACK_ENDED_OK
                modules.postMsg(consts.MSG_EVT_TRACK_ENDED_OK)
            log.c.debug(msg)
        elif msg.type == gst.MESSAGE_ERROR:
            if self.position >= 0:
                self.position = 0
                # MSG_EVT_TRACK_ENDED_ERROR
                modules.postMsg(consts.MSG_EVT_TRACK_ENDED_ERROR)
            log.c.debug(msg)
        #return True
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __queryDuration(self):
        """ Return the duration of the current stream """
        try: return self.player.query_duration(gst.FORMAT_TIME)[0]
        except: return 0

    def __queryPosition(self):
        """ Return the current position """
        try: return self.player.query_position(gst.FORMAT_TIME)[0]
        except: return 0

    def __queryRate(self):
        """ We need the sample rate, get pads() and caps() """
        pads = list(self.player.pads())
        sink = self.player.get_by_name('audiosink')
        pads += list(sink.pads())
        for pad in pads:
            caps = pad.get_negotiated_caps()
            # ['endianness', 'signed', 'width', 'depth', 'rate', 'channels']
            if 'rate' in caps[0].keys():
                return caps[0]['rate']
            else:
                return None

        # if we need to convert gst.FORMAT_BYTES in gst.FORMAT_TIME
        #duration, qformat = self.player.query_duration(gst.FORMAT_TIME)
        #if qformat == gst.FORMAT_BYTES:
        #   duration /= 4
        #if qformat == gst.FORMAT_TIME:
        #    rate = None
        #duration = int(float(rate) * (float(duration) / gst.SECOND))

    def __isPaused(self):
        """ Return whether the player is paused """
        return self.player.get_state()[1] == gst.STATE_PAUSED

    def __isPlaying(self):
        """ Return whether the player is paused """
        return self.player.get_state()[1] == gst.STATE_PLAYING
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __startUpdateTimer(self):
        """ Start the update timer if needed """
        if not self.timer:
            self.timer = gobject.timeout_add(1000, self.updateTimerHandler)

    def __stopUpdateTimer(self):
        """ Start the update timer if needed """
        if self.timer:
            gobject.source_remove(self.timer)
            self.timer = None

    def updateTimerHandler(self):
        """ Regularly called during playback (can be paused) """
        if self.__isPlaying() and not self.seeked:
            self.position = int(self.__queryPosition()/1000000000)
            # MSG_EVT_TRACK_POSITION
            modules.postMsg(consts.MSG_EVT_TRACK_POSITION,
                {'seconds': self.position})
        else:
            self.seeked = False
        #print 163, self.timeout
        return True
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onSeek(self, seconds):#FIXME
        """ Set the new position or queue it """
        if self.__isPlaying(): self.__seek(seconds*1000000000)
        else: self.queueseek = seconds*1000000000
        self.seeked = True

    def __seek(self, where):
        """ Jump to the given secs more 100ms for not seek at 0 """
        self.player.seek(1, gst.FORMAT_TIME,
            gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
            gst.SEEK_TYPE_SET, where +100000000,
            gst.SEEK_TYPE_NONE, 0)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onPlay(self, track):#FIXME
        """ Stop, set the given 'uri' and play it """
        self.uri = track.getURI().replace('%', '%25').replace('#', '%23')
        self.track = track

        self.__stop()
        self.player.set_property('uri', self.uri)
        self.__volume(0)

        # wait 400ms for not fuck gst pipeline
        if self.timeout:
            gobject.source_remove(self.timeout)
        self.timeout = gobject.timeout_add(400, self.__getPlay)

    def __getPlay(self):#FIXME freeze if uri is not playable :(
        """ Wait the buffer of 'uri' and make sure go to playing """
        self.player.set_state(gst.STATE_PLAYING)

        if self.__isPlaying():
            self._level = 0
            
            if self.queueseek: # queued by cmdline#FIXME
                self.__seek(self.queueseek)
                self.queueseek = None

            self.__startUpdateTimer()

            if self.timeout:
                gobject.source_remove(self.timeout)
                self.timeout = None

            if self.timeout:
                gobject.source_remove(self.timeout)
            self.timeout = gobject.timeout_add(100, self.__fadeIn, self.level)

            if not self.uri.startswith('cdda:'):
                self.track = gstrack.getTrack(self.track,
                    self.player.emit("get-audio-tags",0),
                    self.__queryDuration(),
                    self.__queryRate(),
                    self.uri)

            # MSG_EVT_NEW_TRACK
            modules.postMsg(consts.MSG_EVT_NEW_TRACK, {'track': self.track})

        return True

    def onStop(self):
        """ Stop playing """
        self.__stop()
        # MSG_EVT_STOPPED
        modules.postMsg(consts.MSG_EVT_STOPPED)

    def __stop(self):
        """ Stop functions """
        self.player.set_state(gst.STATE_NULL)
        self.__stopUpdateTimer()

        self.position = 0
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onTogglePause(self):
        """ Play/Pause the current 'uri' """
        if self.__isPaused():
            self._level = 0

            self.player.set_state(gst.STATE_PLAYING)

            if self.queueseek: # queued by user
                self.__seek(self.queueseek)
                self.queueseek = None

            if self.timeout:
                gobject.source_remove(self.timeout)
            self.timeout = gobject.timeout_add(100, self.__fadeIn, self.level)
            # MSG_EVT_UNPAUSED
            modules.postMsg(consts.MSG_EVT_UNPAUSED)

        elif self.__isPlaying():
            self._level = self.level

            if self.timeout:
                gobject.source_remove(self.timeout)
            self.timeout = gobject.timeout_add(100, self.__fadeOut, self.level)
            # MSG_EVT_PAUSED
            modules.postMsg(consts.MSG_EVT_PAUSED)

#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onVolumeChanged(self, level):
        """ The volume slider has been changed, wait 200ms before set """
        self.level = round(level, 1)

        if self.timeout:
            gobject.source_remove(self.timeout)
        self.timeout = gobject.timeout_add(400, self.__volume, self.level)

    def __volume(self, level):#FIXME unknow segfault error
        """ Set the volume to the given level """
        if level >= 0.0 and level <= 1.0:
            self.player.set_property('volume', level)

        if level >= self.level or level <= 0.0:
            if self.timeout:
                gobject.source_remove(self.timeout)
                self.timeout = None         

    def __fadeIn(self, level):
        """ Fade in the current volume on play/unpaused """
        self._level += float(level/7)
        self.__volume(self._level)

        return True

    def __fadeOut(self, level):
        """ Fade out the current volume on paused """
        self._level -= float(level/7)
        self.__volume(self._level)

        if self._level <= 0 and self.__isPlaying():
            self.player.set_state(gst.STATE_PAUSED)

        return True
