# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz#

import dbus, modules
from time import time
from tools import consts, log
from gettext import gettext as _

MOD_INFO = ('MediaKeys', _('MediaKeys'), _('Activate the multimedia keys'),
    False, False, 1)

# unique app name so multiple instances not interfere with others
APP_UID = consts.appName + str(time())

class MediaKeys(modules.Module):

    def __init__(self):
        """ Constructor """
        handlers = {
            consts.MSG_EVT_APP_STARTED:     self.onAppStarted,
            consts.MSG_EVT_MOD_LOADED:      self.onAppStarted,
            consts.MSG_EVT_APP_QUIT:        self.onAppQuit,
            consts.MSG_EVT_MOD_UNLOADED:    self.onAppQuit,
            }
        modules.Module.__init__(self, handlers)

    def onAppStarted(self):
        """ The application has started """
        try:
            service = dbus.SessionBus().get_object(
                'org.gnome.SettingsDaemon', '/org/gnome/SettingsDaemon/MediaKeys')
            self.dbusInterface = dbus.Interface(service,
                'org.gnome.SettingsDaemon.MediaKeys')
            self.dbusInterface.GrabMediaPlayerKeys(APP_UID, time())
            self.dbusInterface.connect_to_signal('MediaPlayerKeyPressed',
                self.onMediaKey)
        except Exception, err:
            log.c.error(err)
            self.dbusInterface = None

    def onAppQuit(self):
        """ The application is about to terminate """
        if self.dbusInterface:
            self.dbusInterface.ReleaseMediaPlayerKeys(APP_UID)

    def onMediaKey(self, appName, action):
        """ A media key has been pressed """
        if   action == 'Stop':
            modules.postMsg(consts.MSG_CMD_STOP)
        elif action == 'Next':
            modules.postMsg(consts.MSG_CMD_NEXT)
        elif action == 'Previous':
            modules.postMsg(consts.MSG_CMD_PREVIOUS)
        elif action in ['Play', 'Pause']:
            modules.postMsg(consts.MSG_CMD_TOGGLE_PAUSE)
