# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v2.9~mickyz

import gtk, gobject, modules, os.path
from tools import consts, prefs
from gettext import gettext as _

MOD_INFO = ('Notification', _('Notification'), _('Notifies the track change'),
    False, False, 1)

PREFS_TITLE = '{title} {duration_str}'
PREFS_BODY  = '{artist}'# {album}, {playlist_pos},...
PREFS_TIMEOUT = 4

class Notification(modules.Module):

    def __init__(self):
        """ Constructor """
        handlers = {
            consts.MSG_EVT_APP_STARTED:  self.onModLoaded,
            consts.MSG_EVT_APP_QUIT:     self.hideNotification,
            consts.MSG_EVT_NEW_TRACK:    self.onNewTrack,
            consts.MSG_EVT_NEW_COVER:    self.onNewCover,
            consts.MSG_EVT_STOPPED:      self.hideNotification,
            consts.MSG_EVT_MOD_LOADED:   self.onModLoaded,
            consts.MSG_EVT_MOD_UNLOADED: self.hideNotification,
                   }

        modules.Module.__init__(self, handlers)

    def onModLoaded(self):
        """ The module has been loaded """
        self.notif     = None
        self.cfgWin    = None
        self.timeout   = None
        self.currTrack = None
        self.currCover = consts.pixBuf('cover-frame.png')

    def onNewTrack(self, track):
        """ A new track is being played """
        self.currTrack = track

        if self.timeout:
            gobject.source_remove(self.timeout)
        self.timeout = gobject.timeout_add(700, self.showNotification)

    def onNewCover(self, pixThumbnail, pathFullSize):
        """ Set the cover as icon for the current track """
        self.currCover = pixThumbnail
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __createNotification(self, title, body):
        """ Create the Notification object """
        import pynotify

        if pynotify.init(consts.appNameShort):
            self.notif = pynotify.Notification(title, body)
            self.notif.set_urgency(pynotify.URGENCY_LOW)
            self.notif.set_timeout(
                prefs.get(__name__, 'timeout', PREFS_TIMEOUT) * 1000)

    def showNotification(self):
        """ Show the notification for the current track """
        self.timeout = None

        title = self.currTrack.format(
            prefs.get(__name__, 'title', PREFS_TITLE))
        body = self.currTrack.formatHTMLSafe(
            prefs.get(__name__, 'body', PREFS_BODY))

        if not self.notif:
            self.__createNotification(title, body)
        else:
            self.notif.update(title, body)

        self.notif.set_icon_from_pixbuf(self.currCover)
        self.notif.show()

        return False

    def hideNotification(self):
        """ Hide the notification """
        self.currTrack = None

        if self.timeout:
            gobject.source_remove(self.timeout)
            self.timeout = None

        if self.notif:
            self.notif.close()
# _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def configure(self, parent):
        """ Show the configuration window """
        if self.cfgWin is None:
            import gui

            self.cfgWin = gui.window.Window('Notification.glade', 'vbox1',
                parent, MOD_INFO[1], 320, 160)

            self.cfgWin.getWidget('btn-ok').connect('clicked', self.onBtnOk)

        if not self.cfgWin.isVisible():
            self.cfgWin.getWidget('txt-title').set_text(
                prefs.get(__name__, 'title', PREFS_TITLE))
            self.cfgWin.getWidget('txt-body').set_text(
                prefs.get(__name__, 'body', PREFS_BODY))
            self.cfgWin.getWidget('spn-duration').set_value(
                prefs.get(__name__, 'timeout', PREFS_TIMEOUT))

        self.cfgWin.show()

    def onBtnOk(self, btn):
        """ Save new preferences """
        prefs.set(__name__, 'title',
            self.cfgWin.getWidget('txt-title').get_text())
        prefs.set(__name__, 'body',
            self.cfgWin.getWidget('txt-body').get_text())
        prefs.set(__name__, 'timeout',
            int(self.cfgWin.getWidget('spn-duration').get_value()))

        if self.notif:
            self.notif.set_timeout(
                prefs.get(__name__, 'timeout', PREFS_TIMEOUT) * 1000)

        self.cfgWin.hide()
