# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz #FIXME

import gtk, modules
from tools import consts, prefs
from gettext import gettext as _

MOD_INFO = ('StatusIcon', _('StatusIcon'), _('Add an icon in the system tray'),
    False, False, 1)

class StatusIcon(modules.Module):

    def __init__(self):
        """ Initialize the module """
        handlers = {
            consts.MSG_EVT_APP_STARTED:     self.onAppStarted,
            consts.MSG_EVT_PAUSED:          self.onPaused,
            consts.MSG_EVT_STOPPED:         self.onStopped,
            consts.MSG_EVT_UNPAUSED:        self.onUnpaused,
            consts.MSG_EVT_NEW_TRACK:       self.onNewTrack,
            consts.MSG_EVT_NEW_TRACKLIST:   self.onNewTracklist,
            consts.MSG_EVT_TRACK_MOVED:     self.onTrackMoved,
            consts.MSG_EVT_MOD_LOADED:      self.onModLoaded,
            consts.MSG_EVT_MOD_UNLOADED:    self.onModUnloaded,
                   }
        modules.Module.__init__(self, handlers)

    def onAppStarted(self):
        """ Install the Status Icon """
        self.winDelete      = prefs.get(__name__, 'close-to-tray', True)
        self.isNotPlay      = False
        self.popupMenu      = None
        self.trackHasNext   = False
        self.trackHasPrev   = False
        # get window and connect delete event
        self.window = prefs.getWidgetsTree().get_object('win-main')
        self.window.set_skip_taskbar_hint(self.winDelete)
        self.window.connect('event', self.onDelete)
        # status icon not support RGBA, make sure to use RGB color map
        gtk.widget_push_colormap(self.window.get_screen().get_rgb_colormap())
        self.statusIcon = gtk.StatusIcon()
        gtk.widget_pop_colormap()
        # set icons before connect 'size-changed' event
        self.__setIcons()
        # connect events
        self.statusIcon.connect('activate', lambda w: self.window.present())
        self.statusIcon.connect('popup-menu', self.onPopupMenu)
        self.statusIcon.connect('size-changed', self.__renderIcons)
        # set tooltip
        self.statusIcon.set_tooltip(consts.appName)
        self.statusIcon.set_visible(True)
        # show icon
        self.onNewTrack(None)

    def onModLoaded(self):
        """ The module has been loaded """
        self.onAppStarted()

    def onModUnloaded(self):
        """ Uninstall the Status Icon """
        self.statusIcon.set_visible(False)
        #self.statusIcon = None
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __setIcons(self):
        """ Set the files to load for the tray icons """
        if prefs.get(__name__, 'fixed-icon', True):
            self.trayIcon1 = consts.pixBuf('tray2.png')
            self.trayIcon0 = consts.pixBuf('tray2-notplay.png')
        else:
            self.trayIcon1 = consts.pixBuf('tray.png')
            self.trayIcon0 = consts.pixBuf('tray-notplay.png')

    def __renderIcons(self, statusIcon, availableSize):
        """ Re/Create the tray icons"""
        if self.isNotPlay: statusIcon.set_from_pixbuf(self.trayIcon0)
        else: statusIcon.set_from_pixbuf(self.trayIcon1)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onDelete(self, win, event):
        """ Intercept the close clicks, hide the main window or quit """
        if event.type == gtk.gdk.DELETE and event.send_event and not self.winDelete:
                prefs.getWidgetsTree().get_object('menu-quit').emit('activate')

    def onNewTrack(self, track):
        """ A new track is being played, None if none """
        self.isNotPlay = track is None
        if track:
            self.statusIcon.set_from_pixbuf(self.trayIcon1)
            self.statusIcon.set_tooltip(
                '%s - %s' % (track.getArtist(), track.getTitle()))

    def onNewTracklist(self, tracks, playtime):
        """ A new tracklist has been defined """
        if len(tracks) <= 1:
            self.trackHasNext = False
            self.trackHasPrev = False

    def onTrackMoved(self, hasPrevious, hasNext):
        """ The position of the current track has changed """
        self.trackHasNext = hasNext
        self.trackHasPrev = hasPrevious

    def onPaused(self):
        """ The current track has been paused """
        self.isNotPlay = True
        self.statusIcon.set_from_pixbuf(self.trayIcon0)

    def onUnpaused(self):
        """ The current track has been unpaused """
        self.isNotPlay = False
        self.statusIcon.set_from_pixbuf(self.trayIcon1)

    def onStopped(self):
        """ The playback has been stopped """
        self.isNotPlay = True
        self.onNewTrack(None)
        self.statusIcon.set_from_pixbuf(self.trayIcon0)
        self.statusIcon.set_tooltip(consts.appName)

    def onPopupMenu(self, statusIcon, button, time):
        """ The user asks for the popup menu """
        if self.popupMenu is None:
            from tools import loadGladeFile

            wTree = loadGladeFile('StatusIcon.glade')
            wTree.get_object('item-quit').connect('activate',
                lambda btn: modules.postQuitMsg())

            self.popupMenu  = wTree.get_object('menu-popup')
            self.menuPlay   = wTree.get_object('item-play')
            self.menuNext   = wTree.get_object('item-next')
            self.menuPause  = wTree.get_object('item-pause')
            self.menuPrev   = wTree.get_object('item-previous')

            self.menuPlay.connect('activate',
                lambda btn: modules.postMsg(consts.MSG_CMD_TOGGLE_PAUSE))
            self.menuNext.connect('activate',
                lambda btn: modules.postMsg(consts.MSG_CMD_NEXT))
            self.menuPrev.connect('activate',
                lambda btn: modules.postMsg(consts.MSG_CMD_PREVIOUS))
            self.menuPause.connect('activate',
                lambda btn: modules.postMsg(consts.MSG_CMD_TOGGLE_PAUSE))
            self.popupMenu.show_all()

        self.menuNext.set_sensitive(self.trackHasNext)
        self.menuPrev.set_sensitive(self.trackHasPrev)
        self.menuPause.set_visible(not self.isNotPlay)
        self.menuPlay.set_visible(self.isNotPlay)

        self.popupMenu.popup(None, None,
            gtk.status_icon_position_menu, button, time, statusIcon)
