# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz #FIXME

import gtk, modules, tools, random

from tools import consts, prefs, sec2str
from gettext import gettext as _

MOD_INFO = ('Tracklist', 'Tracklist', '', True, False, None)

(
    ROW_ICO,
    ROW_NUM,
    ROW_TIT,
    ROW_ART,
    ROW_ALB,
    ROW_LEN,
    ROW_BTR,
    ROW_EXT,
    ROW_TRK
) = range(9)

(
    COL_NUM,
    COL_TITLE,
    COL_ARTIST,
    COL_ALBUM,
    COL_LENGTH,
    COL_BITRATE,
    COL_TYPE,
) = range(7)

PREFS_COLUMNS = {
    COL_NUM     : True,
    COL_TITLE   : True,
    COL_ARTIST  : True,
    COL_ALBUM   : False,
    COL_LENGTH  : True,
    COL_BITRATE : False,
    COL_TYPE    : False,
    }

class Tracklist(modules.Module):

    def __init__(self):
        """ Initialization """
        handlers = {
            consts.MSG_EVT_APP_STARTED:     self.onAppStarted,
            consts.MSG_EVT_APP_QUIT:        self.onAppQuit,

            consts.MSG_CMD_NEXT:            self.jumpToNext,
            consts.MSG_CMD_PREVIOUS:        self.jumpToPrevious,
            consts.MSG_CMD_TRACKLIST_PLAY:  self.jumpTo,
            consts.MSG_CMD_TOGGLE_PAUSE:    self.onPause,
            consts.MSG_CMD_TRACKLIST_DEL:   self.remove,
            consts.MSG_CMD_TRACKLIST_ADD:   self.insert,
            consts.MSG_CMD_TRACKLIST_SET:   self.set,
            consts.MSG_CMD_TRACKLIST_CLR: lambda: self.set(None, None),

            consts.MSG_EVT_PAUSED: lambda: self.onPauseToggled(consts.pauseIcon),
            consts.MSG_EVT_UNPAUSED: lambda: self.onPauseToggled(consts.playIcon),
            consts.MSG_EVT_TRACK_ENDED_OK: lambda: self.onTrackEnded(False),
            consts.MSG_EVT_TRACK_ENDED_ERROR:lambda: self.onTrackEnded(True),
            consts.MSG_EVT_STOPPED:         self.onStopped,
            consts.MSG_EVT_NEW_TRACK:       self.onNewTrack,
                   }
        modules.Module.__init__(self, handlers)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onAppStarted(self):
        """ The application has been started """
        self.playtime = 0
        self.previousList = None
        self.lastSearch = None
        self.randList = []
        self.currIdx = None

        self.__setColumns()
        self.__setWidgets()

    def onAppQuit(self):
        """ The application is about to terminate """
        prefs.set(__name__, 'shuffle', self.btnShuffle.get_active())
        prefs.set(__name__, 'repeat', self.btnRepeat.get_active())
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __setWidgets(self):
        """ Set up the widgets of the window """
        wTree = prefs.getWidgetsTree()

        self.window = wTree.get_object('win-main')

        wTree.get_object('scrolled-trklist').add(self.list)

        wTree.get_object('btn-open').connect('clicked', self.onOpenDir)
        wTree.get_object('menu-open').connect('activate', self.onOpenDir)
        wTree.get_object('menu-cd').connect('activate', self.onOpenCD)

        search = wTree.get_object('txt-search')
        search.connect('activate', self.onSearch)
        search.connect('icon-press', self.__delSearch)
        search.connect('focus-out-event', lambda w,e: w.set_text(''))

        self.btnShuffle = wTree.get_object('btn-shuffle')
        self.btnRepeat  = wTree.get_object('btn-repeat')
        self.btnClear   = wTree.get_object('btn-clear')

        self.btnShuffle.set_active(prefs.get(__name__, 'shuffle', False))
        self.btnRepeat.set_active(prefs.get(__name__, 'repeat', False))
        self.btnClear.connect('clicked', lambda w: self.set(None, None))
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __setColumns(self):
        """ Set up the columns' properties """
        from gobject import TYPE_STRING, TYPE_INT, TYPE_PYOBJECT
        from gui.extListview import ExtListView

        txtLRdr   = gtk.CellRendererText()
        txtRRdr   = gtk.CellRendererText()
        pixbufRdr = gtk.CellRendererPixbuf()

        txtLRdr.set_property('width-chars', 100)
        txtLRdr.set_property('ellipsize', 'end')
        txtRRdr.set_property('width-chars', 10)
        txtRRdr.set_property('xalign', 1.0)

        visible = prefs.get(__name__, 'columns', PREFS_COLUMNS)
        columns = (
            ('#', [(pixbufRdr, gtk.gdk.Pixbuf), (txtLRdr, TYPE_INT)], (ROW_NUM,),
            False, visible[COL_NUM]),            
            (_('Title'), [(txtLRdr, TYPE_STRING)], (ROW_TIT,),
            True,  visible[COL_TITLE]),
            (_('Artist'), [(txtLRdr, TYPE_STRING)], (ROW_ART,),
            True,  visible[COL_ARTIST]),
            (_('Album'), [(txtLRdr, TYPE_STRING)], (ROW_ALB,),
            True,  visible[COL_ALBUM]),
            (_('Length'), [(txtRRdr, TYPE_INT)], (ROW_LEN,),
            False, visible[COL_LENGTH]),
            (_('Bitrate'), [(txtRRdr, TYPE_STRING)], (ROW_BTR,),
            False, visible[COL_BITRATE]),
            (_('Type'), [(txtRRdr, TYPE_STRING)], (ROW_EXT,),
            False, visible[COL_TYPE]),
            (None, [(None, TYPE_PYOBJECT)], (None,),
            False, False)
            )

        self.list = ExtListView(columns, sortable=True,        
            dndTargets=consts.DND_TARGETS.values(),
            useMarkup=False, canShowHideColumns=True)

        self.list.get_column(0).set_max_width(20)
        self.list.get_column(1).set_max_width(100)
        self.list.get_column(2).set_max_width(100)
        self.list.get_column(3).set_max_width(100)
        self.list.get_column(4).set_cell_data_func(txtRRdr, self.__fmtLength)
        self.list.get_column(4).set_max_width(60)
        self.list.get_column(5).set_max_width(60)
        self.list.get_column(6).set_max_width(60)

        self.list.enableDNDReordering()

        self.list.connect('extlistview-dnd',
            self.onDND)
        self.list.connect('key-press-event',
            self.onKeyPressed)
        self.list.connect('extlistview-modified',
            self.onListModified)
        self.list.connect('extlistview-button-pressed',
            self.onButtonPressed)
        self.list.connect('extlistview-selection-changed',
            self.onSelectionChanged)
        self.list.connect('extlistview-column-visibility-changed',
            self.onColumnChanged)

    def onColumnChanged(self, list, colTitle, visible):
        """ A column has been shown/hidden """
        if colTitle == '#':            colId = COL_NUM
        elif colTitle == _('Title'):   colId = COL_TITLE
        elif colTitle == _('Artist'):  colId = COL_ARTIST
        elif colTitle == _('Album'):   colId = COL_ALBUM
        elif colTitle == _('Length'):  colId = COL_LENGTH
        elif colTitle == _('Bitrate'): colId = COL_BITRATE
        elif colTitle == _('Type'):    colId = COL_TYPE

        visibility = prefs.get(__name__, 'columns', PREFS_COLUMNS)
        visibility[colId] = visible

        prefs.set(__name__, 'columns', visibility)

    def __fmtLength(self, col, cll, mdl, it):
        """ Format the length text (show 1:00 instead of 60) """
        cll.set_property('text', sec2str(mdl.get_value(it, ROW_LEN)))

    def __getNextTrackIdx(self):
        """ Return the index of the next track, or -1 if there is none """
        if self.list.hasMark():
            if self.list.getMark() < (len(self.list) -1):
                return self.list.getMark() +1
            elif self.btnRepeat.get_active():
                return 0
            elif self.btnShuffle.get_active():
                return 0
        return -1

    def __getPreviousTrackIdx(self):
        """ Return the index of the previous track, or -1 if there is none """
        if self.list.hasMark():
            if self.list.getMark() > 0: return self.list.getMark() -1
            elif self.btnRepeat.get_active(): return len(self.list) -1
            elif self.btnShuffle.get_active(): return len(self.list) -1
        return -1

    def __hasNextTrack(self):
        """ Return whether there is a next track """
        return self.__getNextTrackIdx() != -1

    def __hasPreviousTrack(self):
        """ Return whether there is a previous track """
        return self.__getPreviousTrackIdx() != -1
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def set(self, tracks, playNow):
        """ Replace the tracklist, clear it if tracks is None """
        previousList = [row[ROW_TRK] for row in self.list]
        self.playtime = 0

        self.list.clear()

        if tracks: self.insert(tracks, playNow)
        else: modules.postMsg(consts.MSG_CMD_STOP)

        self.previousList = previousList
        self.randList = []

    def insert(self, tracks, playNow, position=None):
        """ Insert the tracks in the tracklist, append if position is None """
        rows = [[consts.nullIcon,
            track.getPlaylistPos(),
            track.getTitle(),
            track.getArtist(),
            track.getExtendedAlbum(),
            track.getLength(),
            track.getBitrate(),
            track.getType(),
            track] for track in tracks]

        if len(rows) != 0:
            self.previousList = [row[ROW_TRK] for row in self.list]
            self.list.insertRows(rows, position)

            for row in rows: self.playtime += row[ROW_LEN]

            if playNow:
                if position: self.jumpTo(position)
                else: self.jumpTo(len(self.previousList))

    def remove(self, idx=None):
        """ Remove the selected tracks """
        self.currIdx = self.list.getMark()
        
        self.previousList = [row[ROW_TRK] for row in self.list]

        if idx:
            self.playtime -= self.list.getRow(idx)[ROW_LEN]
            self.list.removeRow((idx, ))
        else:
            self.playtime -= sum([row[ROW_LEN]
                for row in self.list.iterSelectedRows()])
            self.list.removeSelectedRows()

        self.list.unselectAll()

        if self.currIdx and not self.list.hasMark():
            self.list.setMark(self.currIdx -1)
            self.jumpToNext()

    def crop(self):
        """ Remove the unselected tracks """
        self.previousList = [row[ROW_TRK] for row in self.list]
        self.playtime = sum([row[ROW_LEN] for row in self.list.iterSelectedRows()])
        self.currIdx = self.list.getMark()

        self.list.cropSelectedRows()
        self.list.unselectAll()

    def revert(self):
        """ Back to the previous tracklist """
        if not self.previousList:
            return

        if self.list.hasMark():
            searchIdx = self.list.getRow(self.list.getMark())[ROW_TRK]
        else:
            searchIdx = None

        self.set(self.previousList, None)
        self.previousList = None

        if searchIdx:
            for i,v in enumerate([row[ROW_TRK] for row in self.list.iterAllRows()]):
                if v == searchIdx:
                    self.currIdx = i
                    break

        if self.currIdx >= 0:
            self.jumpTo(self.currIdx, False)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def __jumpTo(self, where):
        """ Jump to the selected track """
        if where != -1:
            self.jumpTo(where)

    def jumpTo(self, trackIdx, playNow=True):#FIXME
        """ Jump to the track located at the given index """
        if self.list.hasMark():
            if self.list.getItem(self.list.getMark(), ROW_ICO) != consts.errIcon:
                self.list.setItem(self.list.getMark(), ROW_ICO, consts.nullIcon)
                #..set_property('foreground-gdk', style.text[gtk.STATE_INSENSITIVE])

        self.list.setMark(trackIdx)
        self.list.scroll_to_cell(trackIdx)
        self.list.setItem(trackIdx, ROW_ICO, consts.playIcon)

        self.list.unselectAll()
        self.list.selectRow(trackIdx)
    
        self.track = self.list.getItem(trackIdx, ROW_TRK)

        # MSG_CMD_PLAY
        if playNow: modules.postMsg(consts.MSG_CMD_PLAY, {'track': self.track})
        # MSG_EVT_TRACK_MOVED
        modules.postMsg(consts.MSG_EVT_TRACK_MOVED, {
            'hasPrevious': self.__hasPreviousTrack(),
            'hasNext': self.__hasNextTrack()})

    def jumpToNext(self):
        """ Jump to the next track, or random if shuffle is enabled """
        if self.btnShuffle.get_active():
            self.jumpToRandom()
        else: 
            self.__jumpTo(self.__getNextTrackIdx())

    def jumpToPrevious(self):
        """ Jump to the previous track, or previous random """
        if self.btnShuffle.get_active():
            try:
                self.randList.remove(self.randList[-1])
                self.jumpTo(self.randList[-1])
            except:
                self.randList = []

                self.btnRepeat.set_active(False)
                self.btnShuffle.set_active(False)

                self.__jumpTo(self.__getPreviousTrackIdx())
        else: 
            self.__jumpTo(self.__getPreviousTrackIdx())

    def jumpToRandom(self):
        """ Jump to a random track number """
        if not self.randList:
            self.randList.append(self.list.getMark())
        try:
            where = random.choice([ x for x in range(len(self.list))
                if x not in self.randList])

            self.randList.append(where)
            self.__jumpTo(where)
        except:
            self.randList = []

            if not self.btnRepeat.get_active():
                self.btnRepeat.set_active(True)

            self.__jumpTo(0)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onNewTrack(self, track):
        """ A new track is being played """
        rowNum = track.getPlaylistPos() -1

        self.list.setItem(rowNum, ROW_TIT, track.getTitle())
        self.list.setItem(rowNum, ROW_ART, track.getArtist())
        self.list.setItem(rowNum, ROW_ALB, track.getAlbum())
        self.list.setItem(rowNum, ROW_LEN, track.getLength())
        self.list.setItem(rowNum, ROW_BTR, track.getBitrate())

    def onTrackEnded(self, withError):
        """ The current track has ended, jump to the next one or stop """
        trackIdx = self.list.getMark()

        if withError:
            self.list.setItem(trackIdx, ROW_ICO, consts.errIcon)
        if self.btnShuffle.get_active():
            self.jumpToRandom()
            return

        if trackIdx == len(self.list)-1 and not self.btnRepeat.get_active():
            modules.postMsg(consts.MSG_CMD_STOP)
            return

        self.jumpToNext()

    def onPause(self):
        """ Start playing if not already playing """
        if len(self.list) != 0 and not self.list.hasMark():
            if self.list.getSelectedRowsCount() != 0:
                self.jumpTo(self.list.getFirstSelectedRowIndex())
            else:
                self.jumpTo(0)

    def onPauseToggled(self, icon):
        """ Switch between pause and unpause """
        if self.list.hasMark():
            self.list.setItem(self.list.getMark(), ROW_ICO, icon)

    def onStopped(self):
        """ Playback has been stopped """
        if self.list.hasMark():
            currTrack = self.list.getMark()

            if self.list.getItem(currTrack, ROW_ICO) != consts.errIcon:
                self.list.setItem(currTrack, ROW_ICO, consts.nullIcon)

            self.list.clearMark()
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onSearch(self, widget):#FIXME
        """ Search a track in the current playlist """
        currentSearch = widget.get_text()
        widget.set_text('')

        if currentSearch == self.lastSearch or len(currentSearch) <= 2:
            return            
        self.lastSearch = currentSearch

        self.list.unselectAll()

        for i,v in enumerate(self.listTracks):
            #track = '%s - %s' % (v.getArtist(), v.getTitle())
            trackItems = str(v).lower().split(' - ') 
            if [iv for iv in trackItems if currentSearch.lower() in iv] != []:
                self.list.selectRow(i)

        if self.list.getSelectedRowsCount() != 0:
            self.crop()

            widget.set_icon_from_stock(1, gtk.STOCK_REVERT_TO_SAVED)

    def __delSearch(self, widget, icon, event):
        """ Called at icon pressed on search entry """
        if icon == gtk.ENTRY_ICON_SECONDARY:
            widget.set_icon_from_stock(1, None)
            widget.set_text('')

            self.lastSearch = None
            self.revert()
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onOpenCD(self, btn):
        """ Choose a directory to add in the current playlist """
        from media import cdplay

        tracks = cdplay.loadDisc()
        if tracks:
            self.insert(tracks, True)

    def onOpenDir(self, btn):
        """ Choose a directory to add in the current playlist """
        from gui import fileChooser
        from media import getTracks

        tracks = []    
        dirs = fileChooser.openDirectory(self.window, _('Open folder...'))
        if dirs:
            tracks.append(dirs)  
            self.insert(getTracks(tracks), False)

    def onOpenPath(self, path):#FIXME
        """ Open the directory of the selected track """
        import os, subprocess

        rowPath = self.list.getRow(path)[ROW_TRK].getFilePath()
        subprocess.Popen(['xdg-open', os.path.dirname(rowPath)])

    def onEditFile(self, path):#FIXME
        """ Edit the tag information of the selected track """ 
        from media import tag, getTrackFromFile

        rowTrk = self.list.getRow(path)[ROW_TRK]
        rowPth = rowTrk.getFilePath()

        newPth = tag.Tagger(self.window, rowPth)
        if newPth:
            newTrk = getTrackFromFile(newPth)
            rowNum = rowTrk.getPlaylistPos() -1

            self.remove(rowNum)
            self.insert([newTrk,], False, rowNum)

    def onSaveList(self):
        """ Save the current playlist to a file """
        from gui import fileChooser
        from media import savePlaylist

        outFile = fileChooser.save(
            self.window, _('Save playlist'), 'playlist.m3u')
        if outFile:
            allFiles = [row[ROW_TRK].getFilePath() \
                for row in self.list.iterAllRows()]

            savePlaylist(allFiles, outFile)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

    def onDND(self, list, context, x, y, dragData, dndId, time):#FIXME
        """ External Drag'n'Drop """
        import urllib, media

        if dragData.data == '':
            context.finish(False, False, time)
            return

        if dndId == consts.DND_DAP_URI:
            tracks = media.getTracks([urllib.url2pathname(uri) \
            for uri in dragData.data.split()])
        elif dndId == consts.DND_URI:
            tracks = media.getTracks([urllib.url2pathname(uri)[7:] \
            for uri in dragData.data.split()])
        elif dndId == consts.DND_DAP_TRACKS:
            tracks = [media.track.unserialize(serialTrack) \
                for serialTrack in dragData.data.split('\n')]

        dropInfo = list.get_dest_row_at_pos(x, y)
        if not dropInfo:
            self.insert(tracks, False)
        elif dropInfo[1] == gtk.TREE_VIEW_DROP_AFTER:
            self.insert(tracks, False, dropInfo[0][0] + 1)
        else:
            self.insert(tracks, False, dropInfo[0][0])

        context.finish(True, False, time)

    def onListModified(self, list):
        """ Some rows have been added/removed/moved """
        self.btnClear.set_sensitive(len(list) != 0)
        self.btnRepeat.set_sensitive(len(list) != 0)
        self.btnShuffle.set_sensitive(len(list) != 0)

        for position, row in enumerate(self.list):
            row[ROW_TRK].setPlaylistPos(position +1)#FIXME
            row[ROW_TRK].setPlaylistLen(len(self.list))

        self.listTracks = [row[ROW_TRK] for row in self.list]

        # MSG_EVT_NEW_TRACKLIST
        modules.postMsg(consts.MSG_EVT_NEW_TRACKLIST, {
             'tracks': self.listTracks, 'playtime': self.playtime})

        # MSG_EVT_TRACK_MOVED
        if self.list.hasMark():
            modules.postMsg(consts.MSG_EVT_TRACK_MOVED, {
                'hasPrevious': self.__hasPreviousTrack(),
                'hasNext':  self.__hasNextTrack()})

    def onSelectionChanged(self, list, selectedRows):#FIXME
        """ The selection has changed """
        pass

    def onButtonPressed(self, list, event, path):
        """ Play the track on double-click, show a popup menu """
        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS and path:
            self.jumpTo(path[0])
        elif event.button == 3:
            self.onPopupMenu(list, path, event.button, event.time)

    def onPopupMenu(self, list, path, button, time):
        """ Set the menu items """
        popup = gtk.Menu()

        xdgopen = gtk.ImageMenuItem(_('Open path...'))
        xdgopen.set_image(gtk.image_new_from_stock(
            gtk.STOCK_OPEN, gtk.ICON_SIZE_MENU))
        xdgopen.set_sensitive(path != None)
        xdgopen.connect('activate', lambda w: self.onOpenPath(path))
        popup.append(xdgopen)

        popup.append(gtk.SeparatorMenuItem())

        edit = gtk.ImageMenuItem(gtk.STOCK_EDIT)
        edit.set_sensitive(path != None)
        edit.connect("activate", lambda w: self.onEditFile(path))
        popup.append(edit)

        crop = gtk.ImageMenuItem(gtk.STOCK_CUT)
        crop.set_sensitive(path != None)
        crop.connect('activate', lambda w: self.crop())
        popup.append(crop)

        remove = gtk.ImageMenuItem(gtk.STOCK_REMOVE)
        remove.set_sensitive(path != None)
        remove.connect('activate', lambda w: self.remove())
        popup.append(remove)

        popup.append(gtk.SeparatorMenuItem())

        revert = gtk.ImageMenuItem(_('Revert Playlist'))
        revert.set_image(gtk.image_new_from_stock(
            gtk.STOCK_REVERT_TO_SAVED, gtk.ICON_SIZE_MENU))
        revert.set_sensitive(self.previousList != None)
        revert.connect('activate', lambda w: self.revert())
        popup.append(revert)

        clear = gtk.ImageMenuItem(_('Clear Playlist'))
        clear.set_image(gtk.image_new_from_stock(
            gtk.STOCK_CLEAR, gtk.ICON_SIZE_MENU))
        clear.set_sensitive(len(list) != 0)
        clear.connect('activate', lambda w:
            modules.postMsg(consts.MSG_CMD_TRACKLIST_CLR))
        popup.append(clear)

        popup.append(gtk.SeparatorMenuItem())

        save = gtk.ImageMenuItem(_('Save Playlist As...'))
        save.set_image(gtk.image_new_from_stock(
            gtk.STOCK_SAVE_AS, gtk.ICON_SIZE_MENU))
        save.set_sensitive(len(list) != 0)
        save.connect('activate', lambda w: self.onSaveList())
        popup.append(save)

        popup.show_all()
        popup.popup(None, None, None, button, time)

    def onKeyPressed(self, list, event):
        """ Keyboard shortcuts """
        keyname = gtk.gdk.keyval_name(event.keyval)

        if keyname   == 'Delete': self.remove()
        elif keyname == 'Return': self.jumpToNext()
        elif keyname == 'space':  modules.postMsg(consts.MSG_CMD_TOGGLE_PAUSE)
        elif keyname == 'Escape': modules.postMsg(consts.MSG_CMD_STOP)
