# -*- coding: utf-8 -*-
#
# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#
# v3.0~mickyz

import dbus, os.path, sys

(
    PLAY,
    PAUSE,
    NEXT,
    PREVIOUS,
    STOP,
    ADD,
    SET,
    CLEAR,
    VOLUME,
) = range(9)

(CMD_ARGS, CMD_HELP, CMD_NAME) = range(3)

commands = {
    'play':     ( '',               'start playing',             PLAY    ),
    'pause':    ( '',               'pause or continue playing', PAUSE   ),
    'next':     ( '',               'jump to next track',        NEXT    ),
    'prev':     ( '',               'jump to previous track',    PREVIOUS),
    'stop':     ( '',               'stop playback',             STOP    ),
    'add':      ( 'file1 file2...', 'append files to playlist',  ADD     ),
    'set':      ( 'file1 file2...', 'set playlist to given files',SET    ),
    'clear':    ( '',               'clear playlist',            CLEAR   ),
    'volume':   ( 'value (0-100)',  'set volume',                VOLUME  ),
           }

cmdLineOk = False

if len(sys.argv) > 1:
    cmdName = sys.argv[1]

    if cmdName not in commands:
        print '%s is not a valid command'     % cmdName
    elif len(sys.argv) == 2 and commands[cmdName][CMD_ARGS] != '':
        print '%s needs some arguments'       % cmdName
    elif len(sys.argv) > 2 and commands[cmdName][CMD_ARGS] == '':
        print '%s does not take any argument' % cmdName
    else:
        cmdLineOk = True

if not cmdLineOk:
    print 'Usage: decibel-remote [command] [file1 file2...]\n'
    print 'Commands:'
    for cmd, data in sorted(commands.iteritems()):
        print '%s %s %s' % (cmd.ljust(7), data[CMD_ARGS].ljust(14), data[CMD_HELP])
    sys.exit(1)

session = dbus.SessionBus()
activeServices = session.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus').ListNames()

if 'org.mpris.db2' not in activeServices:
    print 'decibel is not running, or d-bus is not available'
    sys.exit(2)

cmd = commands[cmdName][CMD_NAME]

player = dbus.Interface(session.get_object('org.mpris.db2', '/Player'),
    'org.freedesktop.MediaPlayer')

tracklist = dbus.Interface(session.get_object('org.mpris.db2', '/TrackList'),
    'org.freedesktop.MediaPlayer')

if   cmd == SET:      tracklist.SetTracks(sys.argv[2:], True)
elif cmd == ADD:      tracklist.AddTracks(sys.argv[2:], False)
elif cmd == PLAY:     player.Play()
elif cmd == NEXT:     player.Next()
elif cmd == STOP:     player.Stop()
elif cmd == PAUSE:    player.Pause()
elif cmd == CLEAR:    tracklist.Clear()
elif cmd == VOLUME:   player.VolumeSet(int(sys.argv[2]))
elif cmd == PREVIOUS: player.Prev()
