# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz

import gtk, os.path, random, time

# --- Not a constant, but it fits well here
random.seed(int(time.time()))

# --- Descriptions
appName      = 'Decibel² SoundBox'
appComment   = '...Simply Different...'
appVersion   = '2.140~mickyz'
appNameShort = 'decibel'

# --- Directories
dirBaseUsr = os.path.expanduser('~')
dirBaseCfg = os.path.join(dirBaseUsr, '.config')
dirBaseSrc = os.path.join(os.path.dirname(__file__), '..')

dirRes = os.path.join(dirBaseSrc, '..', 'res')
dirDoc = os.path.join(dirBaseSrc, '..', 'doc')
dirPix = os.path.join(dirBaseSrc, '..', 'pix')
dirCfg = os.path.join(dirBaseCfg, appNameShort)

dirLocale = os.path.join(dirBaseSrc, '..', 'locale')
if not os.path.isdir(dirLocale) :
    dirLocale = os.path.join(dirBaseSrc, '..', '..', 'locale')

if not os.path.exists(dirBaseCfg): os.mkdir(dirBaseCfg)
if not os.path.exists(dirCfg): os.mkdir(dirCfg)

# --- Icons
def pixBuf(f): return __pix(f)
def pixPath(f): return os.path.join(dirPix, f)
def __pix(f): return gtk.gdk.pixbuf_new_from_file(pixPath(f))
def __icon(i): return gtk.Label().render_icon(i, gtk.ICON_SIZE_MENU)

appIcon16 = __pix('decibel-16.png')
appIcon24 = __pix('decibel-24.png')
appIcon32 = __pix('decibel-32.png')
appIcon48 = __pix('decibel-48.png')
appIcon96 = __pix('decibel-96.png')
dirIcon   = __icon(gtk.STOCK_DIRECTORY)
errIcon   = __icon(gtk.STOCK_CANCEL)
fileIcon  = __pix('fileaudio.png')
nullIcon  = __pix('filenull.png')
playIcon  = __pix('play.png')
pauseIcon = __pix('pause.png')

# --- Images (loading)
def pixLoad(data):
    return gtk.gdk.pixbuf_new_from_data(
        data.tostring(), # DATA
        gtk.gdk.COLORSPACE_RGB, # COLOR
        True, # IS_RGBA = data.mode=='RGBA' # ALPHA
        8, # BITS
        data.size[0], # WIDTH
        data.size[1], # HEIGTH
        4 * data.size[0]) # if IS_RGBA: 4 else: 3 # ROWSTRIDE

# --- Images (saving)
def pixSave(data, outfile):
    pbl = gtk.gdk.PixbufLoader()
    #pbl.set_size(126, 126)
    pbl.write(data)
    pbuf = pbl.get_pixbuf()
    #pbuf.scale_simple(126, 126, gtk.gdk.INTERP_BILINEAR)
    pbl.close()
    pbuf.save(outfile, 'jpeg', {'quality':'100'})
    #print len(data), os.path.getsize(file)
    #gtk.gdk.pixbuf_new_from_file_at_scale(file, 126, 126, True)

# --- Files
filePrefs    = os.path.join(dirCfg, 'prefs.ini')
filePlaylist = os.path.join(dirCfg, 'playlist.ini')
filePlayed   = os.path.join(dirCfg, 'played.ini')
fileLicense  = os.path.join(dirDoc, 'LICENSE')

# --- D-Bus constants
dbusService   = 'org.mpris.db2'
dbusInterface = 'org.freedesktop.MediaPlayer'

# --- Drag'n'Drop
(
    DND_URI,        # From another application (from Nautilus)
    DND_DAP_URI,    # Inside DAP when tags are not known (from Explorer)
    DND_DAP_TRACKS  # Inside DAP when tags are already known (from Finder)
) = range(3)

DND_TARGETS = {
    DND_URI:        ('text/uri-list',   0,                   DND_URI),
    DND_DAP_URI:    ('dap/uri-list',    gtk.TARGET_SAME_APP, DND_DAP_URI),
    DND_DAP_TRACKS: ('dap/tracks-list', gtk.TARGET_SAME_APP, DND_DAP_TRACKS)
              }

# --- Message that can be sent/received by modules
(
    #_COMMANDS _ _ _ _ _ _ _ _ _#_ _ _ _ _ _ _ _ _ _ _ _#_ _ _ _ _ _ _ _ _ _ _ _
    # GSTPlayer
    MSG_CMD_PLAY,               # Play a resource       # 'uri'
    MSG_CMD_STOP,               # Stop playing          #
    MSG_CMD_SEEK,               # Jump to a position    # 'seconds'
    MSG_CMD_SET_VOLUME,         # Change volume         # 'level'
    MSG_CMD_TOGGLE_PAUSE,       # Toggle play/pause     #
    MSG_CMD_BUFFER_TRACK,       #                       # ''
    # Equalizer
    MSG_CMD_ENABLE_EQZ,         # Enable equalizer      #
    MSG_CMD_SET_EQZ_LVLS,       # Set equalizer levels  # 'lvls'
    # Tracklist
    MSG_CMD_TRACKLIST_PLAY,     # Play given track      # 'idx', 'seconds'
    MSG_CMD_NEXT,               # Play next track       #
    MSG_CMD_PREVIOUS,           # Play previous track   #
    MSG_CMD_TRACKLIST_SET,      # Replace tracklist     # 'tracks', 'playNow'
    MSG_CMD_TRACKLIST_ADD,      # Extend tracklist      # 'tracks', 'playNow'
    MSG_CMD_TRACKLIST_CLR,      # Clear tracklist       #
    MSG_CMD_TRACKLIST_DEL,      # Remove a track        # 'idx'
    # Explorer
    MSG_CMD_INDEX_PATH,         #                       # 'path'
    # Misc
    MSG_CMD_THREAD_EXECUTE,     # Internal thread       # N/A
    #_EVENTS _ _ _ _ _ _ _ _ _ _#_ _ _ _ _ _ _ _ _ _ _ _#_ _ _ _ _ _ _ _ _ _ _ _
    # Application
    MSG_EVT_APP_QUIT,           # Application quit      #
    MSG_EVT_APP_STARTED,        # Application started   #
    # Modules
    MSG_EVT_MOD_LOADED,         # Module loaded         #
    MSG_EVT_MOD_UNLOADED,       # Module unloaded       #
    # Covers
    MSG_EVT_NEW_COVER,          # Cover changed         # 'pixThumb', 'pathFull'
    # GSTPlayer
    MSG_EVT_NEW_TRACK,          # New playing track     # 'track'
    MSG_EVT_TRACK_POSITION,     # Position changed      # 'seconds'
    MSG_EVT_TRACK_ENDED_OK,     # Track ended           #
    MSG_EVT_TRACK_ENDED_ERROR,  # Ended with error      #
    #SG_EVT_TRACK_NEED_BUFFER,  #                       #
    MSG_EVT_PAUSED,             # Paused                #
    MSG_EVT_UNPAUSED,           # Unpaused              #
    MSG_EVT_STOPPED,            # Stopped               #
    # Tracklist
    MSG_EVT_NEW_TRACKLIST,      # New tracklist         # 'tracks', 'playtime'
    MSG_EVT_TRACK_MOVED,        # New track position    # 'hasPrev', 'hasNext'
    # End value
    MSG_END_VALUE
) = range(32)
